@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2011-2014,2016-2017,2019-2020,2022 Aurimas Cernius
+ * Copyright (C) 2011-2014,2016-2017,2019-2020,2022-2023 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -59,25 +59,22 @@ namespace gnote {
 
   bool RemoteControl::AddTagToNote(const Glib::ustring& uri, const Glib::ustring& tag_name)
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(uri);
-    if (!note) {
-      return false;
-    }
-    Tag::Ptr tag = m_manager.tag_manager().get_or_create_tag(tag_name);
-    note->add_tag (tag);
-    return true;
+    return m_manager.find_by_uri(uri, [this, &tag_name](NoteBase & note) {
+      auto tag = m_manager.tag_manager().get_or_create_tag(tag_name);
+      note.add_tag(tag);
+    });
   }
 
 
   Glib::ustring RemoteControl::CreateNamedNote(const Glib::ustring& linked_title)
   {
-    NoteBase::Ptr note = m_manager.find(linked_title);
-    if (note)
+    if(m_manager.find(linked_title)) {
       return "";
+    }
 
     try {
-      note = m_manager.create(Glib::ustring(linked_title));
-      return note->uri();
+      auto & note = m_manager.create(Glib::ustring(linked_title));
+      return note.uri();
     } 
     catch (const std::exception & e) {
       ERR_OUT(_("Exception thrown when creating note: %s"), e.what());
@@ -88,8 +85,7 @@ namespace gnote {
   Glib::ustring RemoteControl::CreateNote()
   {
     try {
-      NoteBase::Ptr note = m_manager.create ();
-      return note->uri();
+      return m_manager.create().uri();
     } 
     catch(...)
     {
@@ -99,40 +95,26 @@ namespace gnote {
 
   bool RemoteControl::DeleteNote(const Glib::ustring& uri)
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(uri);
-    if (!note) {
-      return false;
-    }
-
-    m_manager.delete_note (note);
-    return true;
-
+    return m_manager.find_by_uri(uri, [this](NoteBase & note) {
+      m_manager.delete_note(note);
+    });
   }
 
   bool RemoteControl::DisplayNote(const Glib::ustring& uri)
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(uri);
-    if (!note) {
-      return false;
-    }
-
-    present_note(note);
-    return true;
+    return m_manager.find_by_uri(uri, [this](NoteBase & note) {
+      present_note(note);
+    });
   }
 
 
   bool RemoteControl::DisplayNoteWithSearch(const Glib::ustring& uri, const Glib::ustring& search)
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(uri);
-    if (!note) {
-      return false;
-    }
-
-    MainWindow & window(present_note(note));
-    window.set_search_text(Glib::ustring(search));
-    window.show_search_bar();
-
-    return true;
+    return m_manager.find_by_uri(uri, [this, &search](NoteBase & note) {
+      MainWindow & window(present_note(note));
+      window.set_search_text(Glib::ustring(search));
+      window.show_search_bar();
+    });
   }
 
 
@@ -153,15 +135,18 @@ namespace gnote {
 
   Glib::ustring RemoteControl::FindNote(const Glib::ustring& linked_title)
   {
-    NoteBase::Ptr note = m_manager.find(linked_title);
-    return (!note) ? "" : note->uri();
+    auto note = m_manager.find(linked_title);
+    return (!note) ? "" : note.value().get().uri();
   }
 
 
   Glib::ustring RemoteControl::FindStartHereNote()
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(m_gnote.preferences().start_note_uri());
-    return (!note) ? "" : note->uri();
+    Glib::ustring ret;
+    m_manager.find_by_uri(m_gnote.preferences().start_note_uri(), [&ret](NoteBase & note) {
+      ret = note.uri();
+    });
+    return ret;
   }
 
 
@@ -188,38 +173,41 @@ namespace gnote {
 
   int64_t RemoteControl::GetNoteChangeDateUnix(const Glib::ustring& uri)
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(uri);
-    if (!note)
-      return -1;
-    return note->metadata_change_date().to_unix();
+    int64_t ret = -1;
+    m_manager.find_by_uri(uri, [&ret](NoteBase & note) {
+      ret = note.metadata_change_date().to_unix();
+    });
+    return ret;
   }
 
 
   Glib::ustring RemoteControl::GetNoteCompleteXml(const Glib::ustring& uri)
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(uri);
-    if (!note)
-      return "";
-    return note->get_complete_note_xml();
-
+    Glib::ustring xml;
+    m_manager.find_by_uri(uri, [&xml](NoteBase & note) {
+      xml = note.get_complete_note_xml();
+    });
+    return xml;
   }
 
 
   Glib::ustring RemoteControl::GetNoteContents(const Glib::ustring& uri)
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(uri);
-    if (!note)
-      return "";
-    return note->text_content();
+    Glib::ustring text;
+    m_manager.find_by_uri(uri, [&text](NoteBase & note) {
+      text = note.text_content();
+    });
+    return text;
   }
 
 
   Glib::ustring RemoteControl::GetNoteContentsXml(const Glib::ustring& uri)
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(uri);
-    if (!note)
-      return "";
-    return note->xml_content();
+    Glib::ustring xml;
+    m_manager.find_by_uri(uri, [&xml](NoteBase & note) {
+      xml = note.xml_content();
+    });
+    return xml;
   }
 
 
@@ -231,83 +219,72 @@ namespace gnote {
 
   int64_t RemoteControl::GetNoteCreateDateUnix(const Glib::ustring& uri)
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(uri);
-    if (!note)
-      return -1;
-    return note->create_date().to_unix();
+    int64_t ret = -1;
+    m_manager.find_by_uri(uri, [&ret](NoteBase & note) {
+      ret = note.create_date().to_unix();
+    });
+    return ret;
   }
 
 
   Glib::ustring RemoteControl::GetNoteTitle(const Glib::ustring& uri)
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(uri);
-    if (!note)
-      return "";
-    return note->get_title();
+    Glib::ustring title;
+    m_manager.find_by_uri(uri, [&title](NoteBase & note) {
+      title = note.get_title();
+    });
+    return title;
   }
 
 
   std::vector<Glib::ustring> RemoteControl::GetTagsForNote(const Glib::ustring& uri)
   {
-    NoteBase::Ptr note = m_manager.find_by_uri(uri);
-    if (!note)
-      return std::vector<Glib::ustring>();
-
     std::vector<Glib::ustring> tags;
-    std::vector<Tag::Ptr> l = note->get_tags();
-    for(auto & tag : l) {
-      tags.push_back(tag->normalized_name());
-    }
+    m_manager.find_by_uri(uri, [&tags](NoteBase & note) {
+      for(auto & tag : note.get_tags()) {
+        tags.push_back(tag->normalized_name());
+      }
+    });
     return tags;
   }
 
 
 bool RemoteControl::HideNote(const Glib::ustring& uri)
 {
-  NoteBase::Ptr note = m_manager.find_by_uri(uri);
-  if (!note)
-    return false;
-
-  NoteWindow *window = std::static_pointer_cast<Note>(note)->get_window();
-  if(window == NULL) {
-    return true;
-  }
-  MainWindow *win = MainWindow::get_owning(*window);
-  if(win) {
-    win->unembed_widget(*window);
-  }
-  return true;
+  return m_manager.find_by_uri(uri, [](NoteBase & note) {
+    if(auto window = static_cast<Note&>(note).get_window()) {
+      if(auto win = dynamic_cast<MainWindow*>(window->host())) {
+        win->unembed_widget(*window);
+      }
+    }
+  });
 }
 
 
 std::vector<Glib::ustring> RemoteControl::ListAllNotes()
 {
   std::vector<Glib::ustring> uris;
-  for(const NoteBase::Ptr & iter : m_manager.get_notes()) {
-    uris.push_back(iter->uri());
-  }
+  m_manager.for_each([&uris](const NoteBase & note) {
+    uris.push_back(note.uri());
+  });
   return uris;
 }
 
 
 bool RemoteControl::NoteExists(const Glib::ustring& uri)
 {
-  NoteBase::Ptr note = m_manager.find_by_uri(uri);
-  return note != NULL;
+  return m_manager.find_by_uri(uri).has_value();
 }
 
 
 bool RemoteControl::RemoveTagFromNote(const Glib::ustring& uri, 
                                       const Glib::ustring& tag_name)
 {
-  NoteBase::Ptr note = m_manager.find_by_uri(uri);
-  if (!note)
-    return false;
-  Tag::Ptr tag = m_manager.tag_manager().get_tag(tag_name);
-  if (tag) {
-    note->remove_tag (tag);
-  }
-  return true;
+  return m_manager.find_by_uri(uri, [this, &tag_name](NoteBase & note) {
+    if(auto tag = m_manager.tag_manager().get_tag(tag_name)) {
+      note.remove_tag(tag);
+    }
+  });
 }
 
 
@@ -319,13 +296,10 @@ std::vector<Glib::ustring> RemoteControl::SearchNotes(const Glib::ustring& query
 
   Search search(m_manager);
   std::vector<Glib::ustring> list;
-  Search::ResultsPtr results =
-    search.search_notes(query, case_sensitive, notebooks::Notebook::Ptr());
+  auto results = search.search_notes(query, case_sensitive, notebooks::Notebook::ORef());
 
-  for(Search::Results::const_reverse_iterator iter = results->rbegin();
-      iter != results->rend(); iter++) {
-
-    list.push_back(iter->second->uri());
+  for(auto iter = results.rbegin(); iter != results.rend(); ++iter) {
+    list.push_back(iter->second.get().uri());
   }
 
   return list;
@@ -335,38 +309,27 @@ std::vector<Glib::ustring> RemoteControl::SearchNotes(const Glib::ustring& query
 bool RemoteControl::SetNoteCompleteXml(const Glib::ustring& uri, 
                                        const Glib::ustring& xml_contents)
 {
-  NoteBase::Ptr note = m_manager.find_by_uri(uri);
-  if(!note) {
-    return false;
-  }
-    
-  note->load_foreign_note_xml(xml_contents, CONTENT_CHANGED);
-  return true;
+  return m_manager.find_by_uri(uri, [&xml_contents](NoteBase & note) {
+    note.load_foreign_note_xml(xml_contents, CONTENT_CHANGED);
+  });
 }
 
 
 bool RemoteControl::SetNoteContents(const Glib::ustring& uri, 
                                     const Glib::ustring& text_contents)
 {
-  NoteBase::Ptr note = m_manager.find_by_uri(uri);
-  if(!note) {
-    return false;
-  }
-
-  std::static_pointer_cast<Note>(note)->set_text_content(Glib::ustring(text_contents));
-  return true;
+  return m_manager.find_by_uri(uri, [&text_contents](NoteBase & note) {
+    static_cast<Note&>(note).set_text_content(Glib::ustring(text_contents));
+  });
 }
 
 
 bool RemoteControl::SetNoteContentsXml(const Glib::ustring& uri, 
                                        const Glib::ustring& xml_contents)
 {
-  NoteBase::Ptr note = m_manager.find_by_uri(uri);
-  if(!note) {
-    return false;
-  }
-  note->set_xml_content(Glib::ustring(xml_contents));
-  return true;
+  return m_manager.find_by_uri(uri, [&xml_contents](NoteBase & note) {
+    note.set_xml_content(Glib::ustring(xml_contents));
+  });
 }
 
 
@@ -377,33 +340,27 @@ Glib::ustring RemoteControl::Version()
 
 
 
-void RemoteControl::on_note_added(const NoteBase::Ptr & note)
+void RemoteControl::on_note_added(NoteBase & note)
 {
-  if(note) {
-    NoteAdded(note->uri());
-  }
+  NoteAdded(note.uri());
 }
 
 
-void RemoteControl::on_note_deleted(const NoteBase::Ptr & note)
+void RemoteControl::on_note_deleted(NoteBase & note)
 {
-  if(note) {
-    NoteDeleted(note->uri(), note->get_title());
-  }
+  NoteDeleted(note.uri(), note.get_title());
 }
 
 
-void RemoteControl::on_note_saved(const NoteBase::Ptr & note)
+void RemoteControl::on_note_saved(NoteBase & note)
 {
-  if(note) {
-    NoteSaved(note->uri());
-  }
+  NoteSaved(note.uri());
 }
 
 
-MainWindow & RemoteControl::present_note(const NoteBase::Ptr & note)
+MainWindow & RemoteControl::present_note(NoteBase & note)
 {
-  return *MainWindow::present_default(m_gnote, std::static_pointer_cast<Note>(note));
+  return MainWindow::present_default(m_gnote, static_cast<Note&>(note));
 }
 
 
