@@ -27,6 +27,7 @@
 
 #include <iostream>
 
+#include <adwaita.h>
 #include <glibmm/i18n.h>
 #include <glibmm/stringutils.h>
 #include <glibmm/optionentry.h>
@@ -97,6 +98,7 @@ namespace gnote {
 
   void Gnote::on_startup()
   {
+    adw_init();
     Gtk::Application::on_startup();
     auto display = Gdk::Display::get_default();
     m_icon_theme = Gtk::IconTheme::get_for_display(display);
@@ -153,6 +155,9 @@ namespace gnote {
     m_sync_manager = new sync::SyncManager(*this, default_note_manager());
     m_sync_manager->init();
 
+    m_preferences.signal_color_scheme_changed.connect(sigc::mem_fun(*this, &Gnote::on_color_scheme_pref_changed));
+    on_color_scheme_pref_changed();
+
     m_manager->get_addin_manager().initialize_application_addins();
   }
 
@@ -269,7 +274,7 @@ namespace gnote {
                            "Copyright \xc2\xa9 2009 Hubert Figuiere\n"
                            "Copyright \xc2\xa9 2004-2009 the Tomboy original authors."));
     about->set_comments(_("A simple and easy to use desktop note-taking application."));
-    about->set_website("http://live.gnome.org/Gnote");
+    about->set_website("https://wiki.gnome.org/Apps/Gnote");
     about->set_website_label(_("Homepage"));
     about->set_authors(authors);
     about->set_documenters(documenters);
@@ -282,6 +287,21 @@ namespace gnote {
     about->show();
   }
 
+  void Gnote::on_color_scheme_pref_changed()
+  {
+    auto scheme = m_preferences.color_scheme();
+    auto color_scheme = ADW_COLOR_SCHEME_DEFAULT;
+    if(scheme == Preferences::COLOR_SCHEME_DARK_VAL) {
+      color_scheme = ADW_COLOR_SCHEME_FORCE_DARK;
+    }
+    else if(scheme == Preferences::COLOR_SCHEME_LIGHT_VAL) {
+      color_scheme = ADW_COLOR_SCHEME_FORCE_LIGHT;
+    }
+
+    auto style_manager = adw_style_manager_get_default();
+    g_object_set(style_manager, "color-scheme", color_scheme, nullptr);
+  }
+
   MainWindow & Gnote::new_main_window()
   {
     NoteRecentChanges *win = new NoteRecentChanges(*this, default_note_manager());
@@ -368,7 +388,7 @@ namespace gnote {
       m_sync_dlg->signal_response().connect(sigc::mem_fun(*this, &Gnote::on_sync_dialog_response));
     }
 
-    m_sync_dlg->present();
+    m_sync_dlg->present_ui();
   }
 
 
@@ -438,8 +458,8 @@ namespace gnote {
 
   void Gnote::open_note(const NoteBase & note)
   {
-    auto n = const_cast<NoteBase&>(note).shared_from_this();
-    MainWindow::present_in(get_window_for_note(), std::static_pointer_cast<Note>(n));
+    auto & n = const_cast<NoteBase&>(note);
+    MainWindow::present_in(get_window_for_note(), static_cast<Note&>(n));
   }
 
   notebooks::NotebookManager & Gnote::notebook_manager()
