@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2013 Aurimas Cernius
+ * Copyright (C) 2013,2021 Aurimas Cernius
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,14 +49,6 @@ void EmbeddableWidget::background()
   signal_backgrounded();
 }
 
-void EmbeddableWidget::hint_position(int &, int &)
-{
-}
-
-void EmbeddableWidget::hint_size(int &, int &)
-{
-}
-
 void EmbeddableWidget::size_internals()
 {
 }
