@@ -26,7 +26,6 @@
 
 #include <glibmm/i18n.h>
 #include <gtkmm/button.h>
-#include <gtkmm/stock.h>
 
 #include "ignote.hpp"
 #include "mainwindow.hpp"
@@ -47,65 +46,65 @@ namespace gnote {
 
   namespace noteutils {
 
-    void show_deletion_dialog(const Note::List & notes, Gtk::Window * parent)
+    void show_deletion_dialog(const std::vector<NoteBase::Ref> & notes, Gtk::Window & parent)
     {
       Glib::ustring message;
 
       if(notes.size() == 1) {
         // TRANSLATORS: %1 will be replaced by note title
-        message = Glib::ustring::compose(_("Really delete \"%1\"?"), notes.front()->get_title());
+        message = Glib::ustring::compose(_("Really delete \"%1\"?"), notes.front().get().get_title());
       }
       else {
         // TRANSLATORS: %1 is number of notes
         message = Glib::ustring::compose(ngettext("Really delete %1 note?", "Really delete %1 notes?", notes.size()), notes.size());
       }
 
-      utils::HIGMessageDialog dialog(parent, GTK_DIALOG_DESTROY_WITH_PARENT,
-                                     Gtk::MESSAGE_QUESTION,
-                                     Gtk::BUTTONS_NONE,
-                                     message,
-                                     _("If you delete a note it is permanently lost."));
+      auto dialog = Gtk::make_managed<utils::HIGMessageDialog>(&parent, GTK_DIALOG_DESTROY_WITH_PARENT, Gtk::MessageType::QUESTION,
+                                               Gtk::ButtonsType::NONE, message, _("If you delete a note it is permanently lost."));
 
-      Gtk::Button *button;
+      Gtk::Button *button = Gtk::make_managed<Gtk::Button>(_("_Cancel"), true);
+      dialog->add_action_widget(*button, Gtk::ResponseType::CANCEL);
+      dialog->set_default_response(Gtk::ResponseType::CANCEL);
 
-      button = manage(new Gtk::Button(_("_Cancel"), true));
-      button->property_can_default().set_value(true);
-      button->show ();
-      dialog.add_action_widget(*button, Gtk::RESPONSE_CANCEL);
-      dialog.set_default_response(Gtk::RESPONSE_CANCEL);
-
-      button = manage(new Gtk::Button(_("_Delete"), true));
-      button->property_can_default().set_value(true);
+      button = Gtk::make_managed<Gtk::Button>(_("_Delete"), true);
       button->get_style_context()->add_class("destructive-action");
-      button->show ();
-      dialog.add_action_widget(*button, 666);
+      dialog->add_action_widget(*button, 666);
 
-      int result = dialog.run();
-      if (result == 666) {
-        for(auto & note : notes) {
-          note->manager().delete_note(note);
-        }
+      std::vector<Glib::ustring> note_uris;
+      for(auto & note : notes) {
+        note_uris.push_back(note.get().uri());
       }
-    }
+      dialog->signal_response().connect([&manager=notes.front().get().manager(), dialog, notes=std::move(note_uris)](int result) {
+        if (result == 666) {
+          for(const auto & uri : notes) {
+            if(auto note = manager.find_by_uri(uri)) {
+              manager.delete_note(note.value());
+            }
+          }
+        }
+        dialog->hide();
+      });
 
+      dialog->show();
+    }
   }
 
   namespace {
     
     void show_io_error_dialog (Gtk::Window * parent)
     {
-      utils::HIGMessageDialog dialog(
+      auto dialog = Gtk::make_managed<utils::HIGMessageDialog>(
                               parent,
                               GTK_DIALOG_DESTROY_WITH_PARENT,
-                              Gtk::MESSAGE_ERROR,
-                              Gtk::BUTTONS_OK,
+                              Gtk::MessageType::ERROR,
+                              Gtk::ButtonsType::OK,
                               _("Error saving note data."),
                               _("An error occurred while saving your notes. "
                                 "Please check that you have sufficient disk "
                                 "space, and that you have appropriate rights "
                                 "on ~/.local/share/gnote. Error details can be found in "
                                 "~/.gnote.log."));
-      dialog.run();
+      dialog->show();
     }
 
     void place_cursor_and_selection(const NoteData & data, const Glib::RefPtr<NoteBuffer> & buffer)
@@ -134,7 +133,7 @@ namespace gnote {
   }
 
 
-  const int  NoteData::s_noPosition = -1;
+  const int NoteData::s_noPosition = -1;
 
   NoteData::NoteData(Glib::ustring && _uri)
     : m_uri(std::move(_uri))
@@ -174,7 +173,7 @@ namespace gnote {
     invalidate_text();
   }
 
-  const Glib::ustring & NoteDataBufferSynchronizer::text()
+  const Glib::ustring & NoteDataBufferSynchronizer::text() const
   {
     synchronize_text();
     return data().text();
@@ -261,13 +260,10 @@ namespace gnote {
     for(const auto & iter : m_data.data().tags()) {
       add_tag(iter.second);
     }
-    m_save_timeout = new utils::InterruptableTimeout();
-    m_save_timeout->signal_timeout.connect(sigc::mem_fun(*this, &Note::on_save_timeout));
   }
 
   Note::~Note()
   {
-    delete m_save_timeout;
     delete m_window;
   }
 
@@ -279,7 +275,7 @@ namespace gnote {
     note_data->create_date() = date;
     note_data->set_change_date(date);
       
-    return std::make_shared<Note>(std::move(note_data), std::move(filename), manager, g);
+    return Glib::make_refptr_for_instance(new Note(std::move(note_data), std::move(filename), manager, g));
   }
 
   Note::Ptr Note::create_existing_note(std::unique_ptr<NoteData> data, Glib::ustring && filepath, NoteManager & manager, IGnote & g)
@@ -297,13 +293,12 @@ namespace gnote {
         data->create_date() = d;
       }
     }
-    return std::make_shared<Note>(std::move(data), std::move(filepath), manager, g);
+    return Glib::make_refptr_for_instance(new Note(std::move(data), std::move(filepath), manager, g));
   }
 
   void Note::delete_note()
   {
     m_is_deleting = true;
-    m_save_timeout->cancel ();
     
     // Remove the note from all the tags
     for(NoteData::TagMap::const_iterator iter = m_data.data().tags().begin();
@@ -356,7 +351,7 @@ namespace gnote {
       show_io_error_dialog(dynamic_cast<Gtk::Window*>(m_window->host()));
     }
 
-    signal_saved(shared_from_this());
+    signal_saved(*this);
   }
 
   
@@ -415,35 +410,24 @@ namespace gnote {
     queue_save(NO_CHANGE);
   }
 
-  bool Note::on_window_destroyed(GdkEventAny * /*ev*/)
+  void Note::on_window_destroyed()
   {
     m_window = NULL;
-    return false;
   }
 
   void Note::queue_save (ChangeType changeType)
   {
     DBG_OUT("Got QueueSave");
 
-    // Replace the existing save timeout.  Wait 4 seconds
-    // before saving...
-    m_save_timeout->reset(4000);
-    if (!m_is_deleting)
+    if(!m_is_deleting) {
+      // Replace the existing save timeout.  Wait 4 seconds
+      // before saving...
       m_save_needed = true;
+      static_cast<NoteManager&>(manager()).queue_save(*this);
+    }
     set_change_type(changeType);
   }
 
-  void Note::on_save_timeout()
-  {
-    try {
-      save();
-    }
-    catch(const sharp::Exception &e) 
-    {
-      ERR_OUT(_("Error while saving: %s"), e.what());
-    }
-  }
-
   void Note::remove_tag(Tag & tag)
   {
     Glib::ustring tag_name = tag.normalized_name();
@@ -469,7 +453,7 @@ namespace gnote {
     }
     tag.remove_note(*this);
 
-    signal_tag_removed(shared_from_this(), tag_name);
+    signal_tag_removed(*this, tag_name);
 
     DBG_OUT("Tag removed, queueing save");
     queue_save(OTHER_DATA_CHANGED);
@@ -511,7 +495,7 @@ namespace gnote {
         process_rename_link_update(old_title);
       }
       else {
-        signal_renamed(shared_from_this(), old_title);
+        signal_renamed(*this, old_title);
         queue_save(CONTENT_CHANGED);
       }
     }
@@ -520,66 +504,68 @@ namespace gnote {
 
   void Note::process_rename_link_update(const Glib::ustring & old_title)
   {
-    NoteBase::List linking_notes = manager().get_notes_linking_to(old_title);
-    const Note::Ptr self = std::static_pointer_cast<Note>(shared_from_this());
+    std::vector<NoteBase::Ref> linking_notes(manager().get_notes_linking_to(old_title));
 
     if (!linking_notes.empty()) {
       const NoteRenameBehavior behavior = static_cast<NoteRenameBehavior>(m_gnote.preferences().note_rename_behavior());
 
       if (NOTE_RENAME_ALWAYS_SHOW_DIALOG == behavior) {
-        NoteRenameDialog *dlg = new NoteRenameDialog(linking_notes, old_title, self, m_gnote);
-        dlg->signal_response().connect([this, dlg, old_title, self](int response) {
-          process_rename_link_update_end(response, dlg, old_title, self);
+        NoteRenameDialog *dlg = new NoteRenameDialog(linking_notes, old_title, *this, m_gnote);
+        dlg->signal_response().connect([this, dlg, old_title, self_uri=uri()](int response) {
+          manager().find_by_uri(self_uri, [this, response, dlg, old_title](NoteBase & note) {
+            process_rename_link_update_end(response, dlg, old_title, static_cast<Note&>(note));
+          });
         });
         dlg->present();
         get_window()->editor()->set_editable(false);
       }
       else if (NOTE_RENAME_ALWAYS_REMOVE_LINKS == behavior) {
-        for(NoteBase::Ptr & iter : linking_notes) {
-          iter->remove_links(old_title, self);
-          process_rename_link_update_end(Gtk::RESPONSE_NO, NULL, old_title, self);
+        for(NoteBase & note : linking_notes) {
+          note.remove_links(old_title, *this);
+          process_rename_link_update_end(Gtk::ResponseType::NO, NULL, old_title, *this);
         }
       }
       else if (NOTE_RENAME_ALWAYS_RENAME_LINKS == behavior) {
-        for(NoteBase::Ptr & iter : linking_notes) {
-          iter->rename_links(old_title, self);
-          process_rename_link_update_end(Gtk::RESPONSE_NO, NULL, old_title, self);
+        for(NoteBase & note : linking_notes) {
+          note.rename_links(old_title, *this);
+          process_rename_link_update_end(Gtk::ResponseType::NO, NULL, old_title, *this);
         }
       }
     }
     else {
-      signal_renamed(shared_from_this(), old_title);
+      signal_renamed(*this, old_title);
       queue_save(CONTENT_CHANGED);
     }
   }
 
   void Note::process_rename_link_update_end(int response, Gtk::Dialog *dialog,
-                                            const Glib::ustring & old_title, const Note::Ptr & self)
+                                            const Glib::ustring & old_title, const Note & self)
   {
     if(dialog) {
       NoteRenameDialog *dlg = static_cast<NoteRenameDialog*>(dialog);
       const NoteRenameBehavior selected_behavior = dlg->get_selected_behavior();
-      if(Gtk::RESPONSE_CANCEL != response && NOTE_RENAME_ALWAYS_SHOW_DIALOG != selected_behavior) {
+      if(Gtk::ResponseType::CANCEL != response && NOTE_RENAME_ALWAYS_SHOW_DIALOG != selected_behavior) {
         m_gnote.preferences().note_rename_behavior(selected_behavior);
       }
 
-      const NoteRenameDialog::MapPtr notes = dlg->get_notes();
-
-      for(std::map<NoteBase::Ptr, bool>::const_iterator iter = notes->begin();
-          notes->end() != iter; iter++) {
-        const std::pair<NoteBase::Ptr, bool> p = *iter;
-        if(p.second && response == Gtk::RESPONSE_YES) { // Rename
-          p.first->rename_links(old_title, self);
-        }
-        else {
-          p.first->remove_links(old_title, self);
-        }
-      }
+      const auto notes = dlg->get_notes();
       delete dialog;
+
+      for(const auto & item : notes) {
+        bool rename = item.second && response == Gtk::ResponseType::YES;
+        manager().find_by_uri(item.first, [this, rename, &old_title, &self](NoteBase & note) {
+          if(rename) {
+            note.rename_links(old_title, *this);
+          }
+          else {
+            note.remove_links(old_title, self);
+          }
+        });
+      }
       get_window()->editor()->set_editable(true);
     }
 
-    signal_renamed(shared_from_this(), old_title);
+    signal_renamed(*this, old_title);
     queue_save(CONTENT_CHANGED);
   }
 
@@ -592,9 +578,7 @@ namespace gnote {
   }
 
 
-  void Note::handle_link_rename(const Glib::ustring & old_title,
-                                const NoteBase::Ptr & renamed,
-                                bool rename)
+  void Note::handle_link_rename(const Glib::ustring & old_title, const NoteBase & renamed, bool rename)
   {
     // Check again, things may have changed
     if (!contains_text(old_title))
@@ -619,12 +603,12 @@ namespace gnote {
       else {
         DBG_OUT("Replacing %s with %s",
                 range.text().c_str(),
-                renamed->get_title().c_str());
+                renamed.get_title().c_str());
         const Gtk::TextIter start_iter = range.start();
         const Gtk::TextIter end_iter = range.end();
         m_buffer->erase(start_iter, end_iter);
         m_buffer->insert_with_tag(range.start(),
-                                  renamed->get_title(),
+                                  renamed.get_title(),
                                   link_tag);
       }
     }
@@ -704,8 +688,7 @@ namespace gnote {
   {
     if(!m_window) {
       m_window = new NoteWindow(*this, m_gnote);
-      m_window->signal_delete_event().connect(
-        sigc::mem_fun(*this, &Note::on_window_destroyed));
+      m_window->signal_destroy().connect(sigc::mem_fun(*this, &Note::on_window_destroyed));
 
       m_window->editor()->set_sensitive(enabled());
       if(m_data.data().has_extent()) {
@@ -731,7 +714,7 @@ namespace gnote {
       m_note_window_embedded = true;
     }
 
-    m_gnote.notebook_manager().active_notes_notebook()->add_note(std::static_pointer_cast<Note>(shared_from_this()));
+    m_gnote.notebook_manager().active_notes_notebook().add_note(*this);
   }
 
   void Note::on_note_window_foregrounded()
