@@ -65,7 +65,7 @@ public:
       return m_buffer;
     }
   void set_buffer(Glib::RefPtr<NoteBuffer> && b);
-  const Glib::ustring & text() override;
+  const Glib::ustring & text() const override;
   void set_text(Glib::ustring && t) override;
 
 private:
@@ -89,9 +89,9 @@ class Note
   : public NoteBase
 {
 public:
-  typedef std::shared_ptr<Note> Ptr;
-  typedef std::weak_ptr<Note> WeakPtr;
-  typedef std::vector<Ptr> List;
+  typedef Glib::RefPtr<Note> Ptr;
+  typedef std::reference_wrapper<Note> Ref;
+  typedef std::optional<Ref> ORef;
 
   ~Note();
 
@@ -100,7 +100,6 @@ public:
 
   static Note::Ptr create_existing_note(std::unique_ptr<NoteData> data, Glib::ustring && filepath, NoteManager & manager, IGnote & g);
 
-  Note(std::unique_ptr<NoteData> _data, Glib::ustring && filepath, NoteManager & manager, IGnote & g);
   virtual void delete_note() override;
   static Note::Ptr load(Glib::ustring &&, NoteManager &, IGnote &);
   virtual void save() override;
@@ -157,6 +156,8 @@ protected:
     return m_data;
   }
 private:
+  Note(std::unique_ptr<NoteData> _data, Glib::ustring && filepath, NoteManager & manager, IGnote & g);
+
   bool contains_text(const Glib::ustring & text);
   void handle_link_rename(const Glib::ustring & old_title, const NoteBase & renamed, bool rename) override;
   void on_buffer_changed();
@@ -169,11 +170,10 @@ private:
   void on_buffer_mark_set(const Gtk::TextBuffer::iterator & iter,
                           const Glib::RefPtr<Gtk::TextBuffer::Mark> & insert);
   void on_window_destroyed();
-  void on_save_timeout();
   void process_child_widget_queue();
   void process_rename_link_update(const Glib::ustring & old_title);
   void process_rename_link_update_end(int response, Gtk::Dialog *dialog,
-                                      const Glib::ustring & old_title, const Note::Ptr & self);
+                                      const Glib::ustring & old_title, const Note & self);
   void on_note_window_embedded();
   void on_note_window_foregrounded();
 
@@ -199,7 +199,6 @@ private:
   Glib::RefPtr<NoteBuffer>   m_buffer;
   Glib::RefPtr<NoteTagTable> m_tag_table;
 
-  utils::InterruptableTimeout *m_save_timeout;
   std::queue<ChildWidgetData> m_child_widget_queue;
 
   sigc::signal<void(Note&)> m_signal_opened;
@@ -209,7 +208,7 @@ private:
 };
 
 namespace noteutils {
-  void show_deletion_dialog(const Note::List & notes, Gtk::Window * parent);
+  void show_deletion_dialog(const std::vector<NoteBase::Ref> & notes, Gtk::Window & parent);
 }
 
 }
