@@ -29,6 +29,7 @@
 #include "itagmanager.hpp"
 #include "notebase.hpp"
 #include "notemanagerbase.hpp"
+#include "base/hash.hpp"
 #include "sharp/exception.hpp"
 #include "sharp/files.hpp"
 #include "sharp/map.hpp"
@@ -44,7 +45,7 @@ NoteDataBufferSynchronizerBase::~NoteDataBufferSynchronizerBase()
 {
 }
 
-const Glib::ustring & NoteDataBufferSynchronizerBase::text()
+const Glib::ustring & NoteDataBufferSynchronizerBase::text() const
 {
   return data().text();
 }
@@ -122,7 +123,7 @@ NoteBase::NoteBase(const Glib::ustring && filepath, NoteManagerBase & _manager)
 
 int NoteBase::get_hash_code() const
 {
-  std::hash<std::string> h;
+  Hash<Glib::ustring> h;
   return h(get_title());
 }
 
@@ -156,7 +157,7 @@ void NoteBase::set_title(Glib::ustring && new_title, bool from_user_action)
       process_rename_link_update(old_title);
     }
     else {
-      signal_renamed(shared_from_this(), old_title);
+      signal_renamed(*this, old_title);
       queue_save(CONTENT_CHANGED);
     }
   }
@@ -164,11 +165,11 @@ void NoteBase::set_title(Glib::ustring && new_title, bool from_user_action)
 
 void NoteBase::process_rename_link_update(const Glib::ustring & old_title)
 {
-  for(NoteBase::Ptr & note : m_manager.get_notes_linking_to(old_title)) {
-    note->rename_links(old_title, *this);
+  for(NoteBase & note : m_manager.get_notes_linking_to(old_title)) {
+    note.rename_links(old_title, *this);
   }
 
-  signal_renamed(shared_from_this(), old_title);
+  signal_renamed(*this, old_title);
   queue_save(CONTENT_CHANGED);
 }
 
@@ -178,7 +179,7 @@ void NoteBase::rename_without_link_update(Glib::ustring && newTitle)
     data_synchronizer().data().title() = std::move(newTitle);
 
     // HACK:
-    signal_renamed(shared_from_this(), data_synchronizer().data().title());
+    signal_renamed(*this, data_synchronizer().data().title());
 
     queue_save(CONTENT_CHANGED); // TODO: Right place for this?
   }
@@ -220,7 +221,7 @@ void NoteBase::save()
     ERR_OUT(_("Exception while saving note: %s"), e.what());
   }
 
-  signal_saved(shared_from_this());
+  signal_saved(*this);
 }
 
 void NoteBase::rename_links(const Glib::ustring & old_title, const NoteBase & renamed)
@@ -281,7 +282,7 @@ void NoteBase::remove_tag(Tag & tag)
   thetags.erase(iter);
   tag.remove_note(*this);
 
-  signal_tag_removed(shared_from_this(), tag_name);
+  signal_tag_removed(*this, tag_name);
 
   DBG_OUT("Tag removed, queueing save");
   queue_save(OTHER_DATA_CHANGED);
