@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2014,2017,2019,2022 Aurimas Cernius
+ * Copyright (C) 2010-2014,2017,2019,2022-2023 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -44,13 +44,23 @@ namespace notebooks {
     return s_template_tag;
   }
 
-  bool Notebook::is_template_note(const Note::Ptr & note)
+  bool Notebook::is_template_note(const Note & note)
   {
     Tag::Ptr tag = template_tag();
     if(tag == NULL) {
       return false;
     }
-    return note->contains_tag(tag);
+    return note.contains_tag(tag);
+  }
+
+  Notebook::Ptr Notebook::create(NoteManagerBase& manager, const Glib::ustring& name, bool is_special)
+  {
+    return Glib::make_refptr_for_instance(new Notebook(manager, name, is_special));
+  }
+
+  Notebook::Ptr Notebook::create(NoteManagerBase& manager, const Tag::Ptr& tag)
+  {
+    return Glib::make_refptr_for_instance(new Notebook(manager, tag));
   }
 
   Notebook::Notebook(NoteManagerBase & manager, const Glib::ustring & name, bool is_special)
@@ -109,70 +119,71 @@ namespace notebooks {
   }
 
 
-  Note::Ptr Notebook::find_template_note() const
+  Note::ORef Notebook::find_template_note() const
   {
-    Note::Ptr note;
     Tag::Ptr templ_tag = template_tag();
     Tag::Ptr notebook_tag = m_note_manager.tag_manager().get_system_tag(NOTEBOOK_TAG_PREFIX + get_name());
     if(!templ_tag || !notebook_tag) {
-      return note;
+      return Note::ORef();
     }
     auto notes = templ_tag->get_notes();
     for(NoteBase *n : notes) {
       if(n->contains_tag(notebook_tag)) {
-        note = std::static_pointer_cast<Note>(n->shared_from_this());
-        break;
+        return Note::ORef(std::ref(*static_cast<Note*>(n)));
       }
     }
 
-    return note;
+    return Note::ORef();
   }
 
-  Note::Ptr Notebook::get_template_note() const
+  Note & Notebook::get_template_note() const
   {
-    NoteBase::Ptr note = find_template_note();
-
-    if (!note) {
-      Glib::ustring title = m_default_template_note_title;
-      if(m_note_manager.find(title)) {
-        auto tag_notes = m_tag->get_notes();
-        title = m_note_manager.get_unique_name(title);
+    {
+      auto note = find_template_note();
+      if(note) {
+        return note.value();
       }
-      auto content = NoteManager::get_note_template_content(title);
-      note = m_note_manager.create(std::move(title), std::move(content));
-          
-      // Select the initial text
-      NoteBuffer::Ptr buffer = std::static_pointer_cast<Note>(note)->get_buffer();
-      buffer->select_note_body();
-
-      // Flag this as a template note
-      Tag::Ptr templ_tag = template_tag();
-      note->add_tag(templ_tag);
-
-      // Add on the notebook system tag so Tomboy
-      // will persist the tag/notebook across sessions
-      // if no other notes are added to the notebook.
-      Tag::Ptr notebook_tag = m_note_manager.tag_manager().get_or_create_system_tag(NOTEBOOK_TAG_PREFIX + get_name());
-      note->add_tag (notebook_tag);
-        
-      note->queue_save (CONTENT_CHANGED);
     }
 
-    return std::static_pointer_cast<Note>(note);
+    Glib::ustring title = m_default_template_note_title;
+    if(m_note_manager.find(title)) {
+      auto tag_notes = m_tag->get_notes();
+      title = m_note_manager.get_unique_name(title);
+    }
+    auto content = NoteManager::get_note_template_content(title);
+    auto & note = m_note_manager.create(std::move(title), std::move(content));
+
+    // Select the initial text
+    NoteBuffer::Ptr buffer = static_cast<Note&>(note).get_buffer();
+    buffer->select_note_body();
+
+    // Flag this as a template note
+    Tag::Ptr templ_tag = template_tag();
+    note.add_tag(templ_tag);
+
+    // Add on the notebook system tag so Tomboy
+    // will persist the tag/notebook across sessions
+    // if no other notes are added to the notebook.
+    Tag::Ptr notebook_tag = m_note_manager.tag_manager().get_or_create_system_tag(NOTEBOOK_TAG_PREFIX + get_name());
+    note.add_tag(notebook_tag);
+
+    note.queue_save(CONTENT_CHANGED);
+
+    return static_cast<Note&>(note);
   }
 
-  Note::Ptr Notebook::create_notebook_note()
+  Note & Notebook::create_notebook_note()
   {
     Glib::ustring temp_title;
-    Note::Ptr note_template = get_template_note();
+    auto & note_template = get_template_note();
 
     temp_title = m_note_manager.get_unique_name(_("New Note"));
-    NoteBase::Ptr note = m_note_manager.create_note_from_template(std::move(temp_title), note_template);
+    auto & note = m_note_manager.create_note_from_template(std::move(temp_title), note_template);
 
     // Add the notebook tag
-    note->add_tag(m_tag);
+    note.add_tag(m_tag);
 
-    return std::static_pointer_cast<Note>(note);
+    return static_cast<Note&>(note);
   }
 
   /// <summary>
@@ -184,18 +195,18 @@ namespace notebooks {
   /// <returns>
   /// A <see cref="System.Boolean"/>
   /// </returns>
-  bool Notebook::contains_note(const Note::Ptr & note, bool include_system)
+  bool Notebook::contains_note(const Note & note, bool include_system)
   {
-    bool contains = note->contains_tag(m_tag);
+    bool contains = note.contains_tag(m_tag);
     if(!contains || include_system) {
       return contains;
     }
     return !is_template_note(note);
   }
 
-  bool Notebook::add_note(const Note::Ptr & note)
+  bool Notebook::add_note(Note & note)
   {
-    m_note_manager.notebook_manager().move_note_to_notebook(note, shared_from_this());
+    m_note_manager.notebook_manager().move_note_to_notebook(note, *this);
     return true;
   }
 
