@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2014,2017,2019 Aurimas Cernius
+ * Copyright (C) 2010-2014,2017,2019,2023 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -34,35 +34,47 @@ namespace notebooks {
 /// An object that represents a notebook in Tomboy
 /// </summary>
 class Notebook 
-  : public std::enable_shared_from_this<Notebook>
+  : public Glib::Object
+  , public std::enable_shared_from_this<Notebook>
 {
 public:
-  typedef std::shared_ptr<Notebook> Ptr;
+  typedef Glib::RefPtr<Notebook> Ptr;
+  typedef std::reference_wrapper<Notebook> Ref;
+  typedef std::optional<Ref> ORef;
+
   static const char * NOTEBOOK_TAG_PREFIX;
-  Notebook(NoteManagerBase &, const Glib::ustring &, bool is_special = false);
-  Notebook(NoteManagerBase &, const Tag::Ptr &);
+
+  static Ptr create(NoteManagerBase& manager, const Glib::ustring& name, bool is_special = false);
+  static Ptr create(NoteManagerBase& manager, const Tag::Ptr& tag);
   Glib::ustring get_name() const
     { return m_name; }
   void set_name(const Glib::ustring &);
   virtual Glib::ustring get_normalized_name() const;
   virtual Tag::Ptr    get_tag() const;
-  Note::Ptr find_template_note() const;
-  virtual Note::Ptr   get_template_note() const;
-  Note::Ptr create_notebook_note();
-  virtual bool contains_note(const Note::Ptr & note, bool include_system = false);
-  virtual bool add_note(const Note::Ptr &);
+  Note::ORef find_template_note() const;
+  virtual Note & get_template_note() const;
+  Note & create_notebook_note();
+  virtual bool contains_note(const Note & note, bool include_system = false);
+  virtual bool add_note(Note&);
   static Glib::ustring normalize(const Glib::ustring & s);
 ////
   virtual ~Notebook()
     {}
+  NoteManagerBase & note_manager()
+    {
+      return m_note_manager;
+    }
 protected:
+  Notebook(NoteManagerBase &, const Glib::ustring &, bool is_special = false);
+
   Tag::Ptr template_tag() const;
-  bool is_template_note(const Note::Ptr &);
+  bool is_template_note(const Note&);
 
   NoteManagerBase & m_note_manager;
 private:
   static Tag::Ptr s_template_tag;
 
+  Notebook(NoteManagerBase &, const Tag::Ptr &);
   Notebook(const Notebook &);
   Notebook & operator=(const Notebook &);
   Glib::ustring m_name;
