@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2011-2015,2017,2019,2021-2022 Aurimas Cernius
+ * Copyright (C) 2011-2015,2017,2019,2021-2023 Aurimas Cernius
  * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -57,17 +57,13 @@ namespace gnote {
 
       NoteManager & nm(note_manager());
 
-      for(const NoteBase::Ptr & note : nm.get_notes()) {
-        note->signal_tag_added.connect(
-          sigc::mem_fun(*this, &NotebookApplicationAddin::on_tag_added));
-        note->signal_tag_removed.connect(
-          sigc::mem_fun(*this, &NotebookApplicationAddin::on_tag_removed));
-      }
+      nm.for_each([this](NoteBase & note) {
+        note.signal_tag_added.connect(sigc::mem_fun(*this, &NotebookApplicationAddin::on_tag_added));
+        note.signal_tag_removed.connect(sigc::mem_fun(*this, &NotebookApplicationAddin::on_tag_removed));
+      });
 
       nm.signal_note_added.connect(
         sigc::mem_fun(*this, &NotebookApplicationAddin::on_note_added));
-      nm.signal_note_deleted.connect(
-        sigc::mem_fun(*this, &NotebookApplicationAddin::on_note_deleted));
 
       am.add_app_action("new-notebook");
       am.get_app_action("new-notebook")->signal_activate().connect(
@@ -90,16 +86,14 @@ namespace gnote {
 
     void NotebookApplicationAddin::on_new_notebook_action(const Glib::VariantBase&)
     {
-      NotebookManager::prompt_create_new_notebook(ignote(), ignote().get_main_window());
+      auto & gnote = ignote();
+      gnote.notebook_manager().prompt_create_new_notebook(gnote, gnote.get_main_window());
     }
 
 
     void NotebookApplicationAddin::on_tag_added(const NoteBase & note, const Tag::Ptr& tag)
     {
       NotebookManager & manager = ignote().notebook_manager();
-      if (manager.is_adding_notebook()) {
-        return;
-      }
 
       Glib::ustring megaPrefix(Tag::SYSTEM_TAG_PREFIX);
       megaPrefix += Notebook::NOTEBOOK_TAG_PREFIX;
@@ -109,14 +103,14 @@ namespace gnote {
 
       Glib::ustring notebookName = sharp::string_substring(tag->name(), megaPrefix.size());
 
-      Notebook::Ptr notebook = manager.get_or_create_notebook(notebookName);
+      auto & notebook = manager.get_or_create_notebook(notebookName);
 
-      manager.signal_note_added_to_notebook() (static_cast<const Note&>(note), notebook);
+      manager.signal_note_added_to_notebook(static_cast<const Note&>(note), notebook);
     }
 
 
 
-    void NotebookApplicationAddin::on_tag_removed(const NoteBase::Ptr & note,
+    void NotebookApplicationAddin::on_tag_removed(const NoteBase & note,
                                                   const Glib::ustring & normalizedTagName)
     {
       Glib::ustring megaPrefix(Tag::SYSTEM_TAG_PREFIX);
@@ -130,28 +124,22 @@ namespace gnote {
         sharp::string_substring(normalizedTagName, megaPrefix.size());
 
       NotebookManager & manager = ignote().notebook_manager();
-      Notebook::Ptr notebook = manager.get_notebook(normalizedNotebookName);
-      if (!notebook) {
+      auto nb = manager.get_notebook(normalizedNotebookName);
+      if(!nb) {
         return;
       }
 
-      manager.signal_note_removed_from_notebook() (*std::static_pointer_cast<Note>(note), notebook);
+      Notebook & notebook = nb.value();
+      manager.signal_note_removed_from_notebook(static_cast<const Note&>(note), notebook);
     }
 
-    void NotebookApplicationAddin::on_note_added(const NoteBase::Ptr & note)
+    void NotebookApplicationAddin::on_note_added(NoteBase & note)
     {
-        note->signal_tag_added.connect(
+        note.signal_tag_added.connect(
           sigc::mem_fun(*this, &NotebookApplicationAddin::on_tag_added));
-        note->signal_tag_removed.connect(
+        note.signal_tag_removed.connect(
           sigc::mem_fun(*this, &NotebookApplicationAddin::on_tag_removed));
     }
 
-
-    void NotebookApplicationAddin::on_note_deleted(const NoteBase::Ptr &)
-    {
-      // remove the signal to the note...
-    }
-
-
   }
 }
