@@ -28,6 +28,7 @@
 #include "utils.hpp"
 #include "trie.hpp"
 #include "notebooks/notebookmanager.hpp"
+#include "base/hash.hpp"
 #include "sharp/directory.hpp"
 #include "sharp/files.hpp"
 #include "sharp/string.hpp"
@@ -36,31 +37,24 @@
 
 namespace gnote {
 
-bool compare_dates(const NoteBase::Ptr & a, const NoteBase::Ptr & b)
-{
-  return (std::static_pointer_cast<Note>(a)->change_date() > std::static_pointer_cast<Note>(b)->change_date());
-}
-
-
 class TrieController
 {
 public:
   TrieController(NoteManagerBase &);
-  ~TrieController();
 
   void add_note(const NoteBase::Ptr & note);
   void update();
-  TrieTree<NoteBase::WeakPtr> *title_trie() const
+  TrieTree<Glib::ustring> & title_trie() const
     {
-      return m_title_trie;
+      return *m_title_trie;
     }
 private:
-  void on_note_added(const NoteBase::Ptr & added);
-  void on_note_deleted (const NoteBase::Ptr & deleted);
-  void on_note_renamed(const NoteBase::Ptr & renamed, const Glib::ustring & old_title);
+  void on_note_added(NoteBase & added);
+  void on_note_deleted(NoteBase & deleted);
+  void on_note_renamed(const NoteBase & renamed, const Glib::ustring & old_title);
 
   NoteManagerBase & m_manager;
-  TrieTree<NoteBase::WeakPtr> *m_title_trie;
+  std::unique_ptr<TrieTree<Glib::ustring>> m_title_trie;
 };
 
 
@@ -160,30 +154,28 @@ TrieController *NoteManagerBase::create_trie_controller()
 
 void NoteManagerBase::post_load()
 {
-  std::sort(m_notes.begin(), m_notes.end(), compare_dates);
-
   // Update the trie so addins can access it, if they want.
   m_trie_controller->update ();
 }
 
 size_t NoteManagerBase::trie_max_length()
 {
-  return m_trie_controller->title_trie()->max_length();
+  return m_trie_controller->title_trie().max_length();
 }
 
-TrieHit<NoteBase::WeakPtr>::ListPtr NoteManagerBase::find_trie_matches(const Glib::ustring & match)
+TrieHit<Glib::ustring>::List NoteManagerBase::find_trie_matches(const Glib::ustring & match)
 {
-  return m_trie_controller->title_trie()->find_matches(match);
+  return m_trie_controller->title_trie().find_matches(match);
 }
 
-NoteBase::List NoteManagerBase::get_notes_linking_to(const Glib::ustring & title) const
+std::vector<NoteBase::Ref> NoteManagerBase::get_notes_linking_to(const Glib::ustring & title) const
 {
   Glib::ustring tag = "<link:internal>" + utils::XmlEncoder::encode(title) + "</link:internal>";
-  NoteBase::List result;
+  std::vector<NoteBase::Ref> result;
   for(const NoteBase::Ptr & note : m_notes) {
     if(note->get_title() != title) {
       if(note->get_complete_note_xml().find(tag) != Glib::ustring::npos) {
-        result.push_back(note);
+        result.push_back(*note);
       }
     }
   }
@@ -195,81 +187,78 @@ void NoteManagerBase::add_note(NoteBase::Ptr note)
   if(note) {
     note->signal_renamed.connect(sigc::mem_fun(*this, &NoteManagerBase::on_note_rename));
     note->signal_saved.connect(sigc::mem_fun(*this, &NoteManagerBase::on_note_save));
-    m_notes.push_back(std::move(note));
+    m_notes.insert(std::move(note));
   }
 }
 
-void NoteManagerBase::on_note_rename(const NoteBase::Ptr & note, const Glib::ustring & old_title)
+void NoteManagerBase::on_note_rename(const NoteBase & note, const Glib::ustring & old_title)
 {
   signal_note_renamed(note, old_title);
-  std::sort(m_notes.begin(), m_notes.end(), compare_dates);
 }
 
-void NoteManagerBase::on_note_save (const NoteBase::Ptr & note)
+void NoteManagerBase::on_note_save(NoteBase & note)
 {
   signal_note_saved(note);
-  std::sort(m_notes.begin(), m_notes.end(), compare_dates);
 }
 
-NoteBase::Ptr NoteManagerBase::find(const Glib::ustring & linked_title) const
+NoteBase::ORef NoteManagerBase::find(const Glib::ustring & linked_title) const
 {
   for(const NoteBase::Ptr & note : m_notes) {
     if(note->get_title().lowercase() == linked_title.lowercase()) {
-      return note;
+      return std::ref(*note);
     }
   }
-  return NoteBase::Ptr();
+  return NoteBase::ORef();
 }
 
-NoteBase::Ptr NoteManagerBase::find_by_uri(const Glib::ustring & uri) const
+NoteBase::ORef NoteManagerBase::find_by_uri(const Glib::ustring & uri) const
 {
   for(const NoteBase::Ptr & note : m_notes) {
     if (note->uri() == uri) {
-      return note;
+      return std::ref(*note);
     }
   }
-  return NoteBase::Ptr();
+  return NoteBase::ORef();
 }
 
-NoteBase::Ptr NoteManagerBase::create_note_from_template(Glib::ustring && title, const NoteBase::Ptr & template_note)
+NoteBase & NoteManagerBase::create_note_from_template(Glib::ustring && title, const NoteBase & template_note)
 {
   return create_note_from_template(std::move(title), template_note, "");
 }
 
-NoteBase::Ptr NoteManagerBase::create()
+NoteBase & NoteManagerBase::create()
 {
   return create_note("", "");
 }
 
-NoteBase::Ptr NoteManagerBase::create(Glib::ustring && title)
+NoteBase & NoteManagerBase::create(Glib::ustring && title)
 {
   Glib::ustring body;
   auto note_title = split_title_from_content(title, body);
   return create_note(std::move(note_title), std::move(body));
 }
 
-NoteBase::Ptr NoteManagerBase::create(Glib::ustring && title, Glib::ustring && xml_content)
+NoteBase & NoteManagerBase::create(Glib::ustring && title, Glib::ustring && xml_content)
 {
   return create_new_note(std::move(title), std::move(xml_content), "");
 }
 
 // Creates a new note with the given title and guid with body based on
 // the template note.
-NoteBase::Ptr NoteManagerBase::create_note_from_template(Glib::ustring && title, const NoteBase::Ptr & template_note, Glib::ustring && guid)
+NoteBase & NoteManagerBase::create_note_from_template(Glib::ustring && title, const NoteBase & template_note, Glib::ustring && guid)
 {
   Tag::Ptr template_save_title = tag_manager().get_or_create_system_tag(ITagManager::TEMPLATE_NOTE_SAVE_TITLE_SYSTEM_TAG);
-  if(template_note->contains_tag(template_save_title)) {
-    title = get_unique_name(template_note->get_title());
+  if(template_note.contains_tag(template_save_title)) {
+    title = get_unique_name(template_note.get_title());
   }
 
   // Use the body from the template note
-  Glib::ustring xml_content = sharp::string_replace_first(template_note->xml_content(),
-                                                          utils::XmlEncoder::encode(template_note->get_title()),
+  Glib::ustring xml_content = sharp::string_replace_first(template_note.xml_content(),
+                                                          utils::XmlEncoder::encode(template_note.get_title()),
                                                           utils::XmlEncoder::encode(title));
   xml_content = sanitize_xml_content(xml_content);
 
-  NoteBase::Ptr new_note = create_new_note(std::move(title), std::move(xml_content), std::move(guid));
-  return new_note;
+  return create_new_note(std::move(title), std::move(xml_content), std::move(guid));
 }
 
 // Find a title that does not exist using basename
@@ -287,7 +276,7 @@ Glib::ustring NoteManagerBase::get_unique_name(const Glib::ustring & basename) c
   return title;
 }
 
-NoteBase::Ptr NoteManagerBase::create_note(Glib::ustring && title, Glib::ustring && body, Glib::ustring && guid)
+NoteBase & NoteManagerBase::create_note(Glib::ustring && title, Glib::ustring && body, Glib::ustring && guid)
 {
   if(title.empty()) {
     title = get_unique_name(_("New Note"));
@@ -295,9 +284,8 @@ NoteBase::Ptr NoteManagerBase::create_note(Glib::ustring && title, Glib::ustring
 
   Glib::ustring content;
   if(body.empty()) {
-    auto template_note = find_template_note();
-    if(template_note) {
-      return create_note_from_template(std::move(title), template_note, std::move(guid));
+    if(auto template_note = find_template_note()) {
+      return create_note_from_template(std::move(title), *template_note, std::move(guid));
     }
 
     // Use a simple "Describe..." body and highlight
@@ -312,7 +300,7 @@ NoteBase::Ptr NoteManagerBase::create_note(Glib::ustring && title, Glib::ustring
 }
 
 // Create a new note with the specified Xml content
-NoteBase::Ptr NoteManagerBase::create_new_note(Glib::ustring && title, Glib::ustring && xml_content, Glib::ustring && guid)
+NoteBase & NoteManagerBase::create_new_note(Glib::ustring && title, Glib::ustring && xml_content, Glib::ustring && guid)
 {
   if(title.empty())
     throw sharp::Exception("Invalid title");
@@ -334,11 +322,11 @@ NoteBase::Ptr NoteManagerBase::create_new_note(Glib::ustring && title, Glib::ust
   new_note->signal_renamed.connect(sigc::mem_fun(*this, &NoteManagerBase::on_note_rename));
   new_note->signal_saved.connect(sigc::mem_fun(*this, &NoteManagerBase::on_note_save));
 
-  m_notes.push_back(new_note);
+  m_notes.insert(new_note);
 
-  signal_note_added(new_note);
+  signal_note_added(*new_note);
 
-  return new_note;
+  return *new_note;
 }
 
 Glib::ustring NoteManagerBase::get_note_template_content(const Glib::ustring & title)
@@ -356,27 +344,24 @@ Glib::ustring NoteManagerBase::get_note_content(const Glib::ustring & title, con
              utils::XmlEncoder::encode(body));
 }
 
-NoteBase::Ptr NoteManagerBase::get_or_create_template_note()
+NoteBase & NoteManagerBase::get_or_create_template_note()
 {
-  NoteBase::Ptr template_note = find_template_note();
-  if(!template_note) {
-    Glib::ustring title = m_default_note_template_title;
-    if(find(title)) {
-      title = get_unique_name(title);
-    }
-    auto content = get_note_template_content(title);
-    template_note = create(std::move(title), std::move(content));
-    if(template_note == 0) {
-      throw sharp::Exception("Failed to create template note");
-    }
-
-    // Flag this as a template note
-    Tag::Ptr template_tag = tag_manager().get_or_create_system_tag(ITagManager::TEMPLATE_NOTE_SYSTEM_TAG);
-    template_note->add_tag(template_tag);
-
-    template_note->queue_save(CONTENT_CHANGED);
+  if(auto template_note = find_template_note()) {
+    return template_note.value();
   }
-      
+
+  Glib::ustring title = m_default_note_template_title;
+  if(find(title)) {
+    title = get_unique_name(title);
+  }
+  auto content = get_note_template_content(title);
+  auto & template_note = create(std::move(title), std::move(content));
+
+  // Flag this as a template note
+  Tag::Ptr template_tag = tag_manager().get_or_create_system_tag(ITagManager::TEMPLATE_NOTE_SYSTEM_TAG);
+  template_note.add_tag(template_tag);
+
+  template_note.queue_save(CONTENT_CHANGED);
   return template_note;
 }
 
@@ -417,37 +402,35 @@ Glib::ustring NoteManagerBase::make_new_file_name(const Glib::ustring & guid) co
   return Glib::build_filename(notes_dir(), guid + ".note");
 }
 
-NoteBase::Ptr NoteManagerBase::find_template_note() const
+NoteBase::ORef NoteManagerBase::find_template_note() const
 {
-  NoteBase::Ptr template_note;
-  Tag::Ptr template_tag = tag_manager().get_system_tag(ITagManager::TEMPLATE_NOTE_SYSTEM_TAG);
-  if(!template_tag) {
-    return template_note;
-  }
-  auto notes = template_tag->get_notes();
-  for(NoteBase *iter : notes) {
-    NoteBase::Ptr note = iter->shared_from_this();
-    if(!m_gnote.notebook_manager().get_notebook_from_note(note)) {
-      template_note = note;
-      break;
+  if(auto template_tag = tag_manager().get_system_tag(ITagManager::TEMPLATE_NOTE_SYSTEM_TAG)) {
+    auto notes = template_tag->get_notes();
+    for(NoteBase *iter : notes) {
+      if(!m_gnote.notebook_manager().get_notebook_from_note(*iter)) {
+        return std::ref(*iter);
+      }
     }
   }
 
-  return template_note;
+  return NoteBase::ORef();
 }
 
 void NoteManagerBase::delete_note(NoteBase & note)
 {
   DBG_OUT("Deleting note '%s'.", note.get_title().c_str());
+  NoteBase::Ptr cached_ref;  // prevent note from being destroyed
 
   for(auto iter = m_notes.begin(); iter != m_notes.end(); ++iter) {
     if(iter->get() == &note) {
+      cached_ref = *iter;
       m_notes.erase(iter);
       break;
     }
   }
+  DBG_ASSERT(cached_ref != nullptr, "Deleting note that is not present");
   note.delete_note();
-  signal_note_deleted(note.shared_from_this());
+  signal_note_deleted(note);
 
   auto file_path = note.file_path();
   if(sharp::file_exists(file_path)) {
@@ -469,7 +452,7 @@ void NoteManagerBase::delete_note(NoteBase & note)
   }
 }
 
-NoteBase::Ptr NoteManagerBase::import_note(const Glib::ustring & file_path)
+NoteBase::ORef NoteManagerBase::import_note(const Glib::ustring & file_path)
 {
   Glib::ustring dest_file = Glib::build_filename(notes_dir(), 
                                                  sharp::file_filename(file_path));
@@ -477,22 +460,35 @@ NoteBase::Ptr NoteManagerBase::import_note(const Glib::ustring & file_path)
   if(sharp::file_exists(dest_file)) {
     dest_file = make_new_file_name();
   }
-  NoteBase::Ptr note;
   try {
     sharp::file_copy(file_path, dest_file);
 
-    // TODO: make sure the title IS unique.
-    note = note_load(std::move(dest_file));
+    NoteBase::Ptr note = note_load(std::move(dest_file));
+    if(!note) {
+      return NoteBase::ORef();
+    }
+
+    if(find(note->get_title())) {
+      for(int i = 1;; ++i) {
+        auto new_title = note->get_title() + " " + TO_STRING(i);
+        if(!find(new_title)) {
+          note->set_title(std::move(new_title));
+          break;
+        }
+      }
+    }
     add_note(note);
+    return *note;
   }
   catch(...)
   {
   }
-  return note;
+
+  return NoteBase::ORef();
 }
 
 
-NoteBase::Ptr NoteManagerBase::create_with_guid(Glib::ustring && title, Glib::ustring && guid)
+NoteBase & NoteManagerBase::create_with_guid(Glib::ustring && title, Glib::ustring && guid)
 {
   Glib::ustring body;
   auto note_title = split_title_from_content(title, body);
@@ -500,10 +496,16 @@ NoteBase::Ptr NoteManagerBase::create_with_guid(Glib::ustring && title, Glib::us
 }
 
 
+std::size_t NoteManagerBase::NoteHash::operator()(const NoteBase::Ptr & note) const noexcept
+{
+  Hash<Glib::ustring> hash;
+  return hash(note->uri());
+}
+
+
 
 TrieController::TrieController(NoteManagerBase & manager)
   : m_manager(manager)
-  ,  m_title_trie(NULL)
 {
   m_manager.signal_note_deleted.connect(sigc::mem_fun(*this, &TrieController::on_note_deleted));
   m_manager.signal_note_added.connect(sigc::mem_fun(*this, &TrieController::on_note_added));
@@ -512,42 +514,34 @@ TrieController::TrieController(NoteManagerBase & manager)
   update();
 }
 
-TrieController::~TrieController()
+void TrieController::on_note_added(NoteBase & note)
 {
-  delete m_title_trie;
+  add_note(note.shared_from_this());
 }
 
-void TrieController::on_note_added(const NoteBase::Ptr & note)
-{
-  add_note(note);
-}
-
-void TrieController::on_note_deleted(const NoteBase::Ptr &)
+void TrieController::on_note_deleted(NoteBase &)
 {
   update();
 }
 
-void TrieController::on_note_renamed(const NoteBase::Ptr &, const Glib::ustring &)
+void TrieController::on_note_renamed(const NoteBase &, const Glib::ustring &)
 {
   update();
 }
 
 void TrieController::add_note(const NoteBase::Ptr & note)
 {
-  m_title_trie->add_keyword(note->get_title(), note);
+  m_title_trie->add_keyword(note->get_title(), note->uri());
   m_title_trie->compute_failure_graph();
 }
 
 void TrieController::update()
 {
-  if(m_title_trie) {
-    delete m_title_trie;
-  }
-  m_title_trie = new TrieTree<NoteBase::WeakPtr>(false /* !case_sensitive */);
+  m_title_trie = std::make_unique<TrieTree<Glib::ustring>>(false /* !case_sensitive */);
 
-  for(const NoteBase::Ptr & note : m_manager.get_notes()) {
-    m_title_trie->add_keyword(note->get_title(), note);
-  }
+  m_manager.for_each([this](NoteBase & note) {
+    m_title_trie->add_keyword(note.get_title(), note.uri());
+  });
   m_title_trie->compute_failure_graph();
 }
 
