@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2011-2023 Aurimas Cernius
+ * Copyright (C) 2011-2024 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -326,8 +326,7 @@ namespace gnote {
     popover_widgets.push_back(PopoverWidget(NOTE_SECTION_FLAGS, IMPORTANT_ORDER, important));
 
     NoteManager & manager = static_cast<NoteManager&>(m_note.manager());
-    Note::Ptr note = std::dynamic_pointer_cast<Note>(m_note.shared_from_this());
-    for(NoteAddin *addin : manager.get_addin_manager().get_note_addins(note)) {
+    for(NoteAddin *addin : manager.get_addin_manager().get_note_addins(m_note)) {
       auto addin_widgets = addin->get_actions_popover_widgets();
       popover_widgets.insert(popover_widgets.end(), addin_widgets.begin(), addin_widgets.end());
     }
@@ -344,9 +343,11 @@ namespace gnote {
   void NoteWindow::on_delete_button_clicked(const Glib::VariantBase&)
   {
     // Prompt for note deletion
-    Note::List single_note_list;
-    single_note_list.push_back(std::static_pointer_cast<Note>(m_note.shared_from_this()));
-    noteutils::show_deletion_dialog(single_note_list, dynamic_cast<Gtk::Window*>(host()));
+    if(auto host_window = dynamic_cast<Gtk::Window*>(host())) {
+      std::vector<NoteBase::Ref> single_note_list;
+      single_note_list.push_back(m_note);
+      noteutils::show_deletion_dialog(single_note_list, *host_window);
+    }
   }
 
   Glib::RefPtr<Gio::MenuModel> NoteWindow::editor_extra_menu()
@@ -450,7 +451,7 @@ namespace gnote {
   }
 
 
-  void NoteWindow::on_note_tag_removed(const NoteBase::Ptr&, const Glib::ustring & tag)
+  void NoteWindow::on_note_tag_removed(const NoteBase&, const Glib::ustring & tag)
   {
     if(tag == m_template_tag->normalized_name()) {
       m_template_widget->hide();
@@ -475,10 +476,10 @@ namespace gnote {
     if (title.empty())
       return;
 
-    NoteBase::Ptr match = m_note.manager().find(title);
-    if (!match) {
+    auto match = m_note.manager().find(title);
+    if(!match) {
       try {
-        match = m_note.manager().create(std::move(select));
+        match = NoteBase::Ref(std::ref(m_note.manager().create(std::move(select))));
       } 
       catch (const sharp::Exception & e) {
         auto dialog = Gtk::make_managed<utils::HIGMessageDialog>(dynamic_cast<Gtk::Window*>(host()),
@@ -496,7 +497,7 @@ namespace gnote {
       m_note.get_buffer()->apply_tag(m_note.get_tag_table()->get_link_tag(), start, end);
     }
 
-    MainWindow::present_in(*dynamic_cast<MainWindow*>(host()), std::static_pointer_cast<Note>(match));
+    MainWindow::present_in(*dynamic_cast<MainWindow*>(host()), static_cast<Note&>(match.value().get()));
   }
 
   bool NoteWindow::open_help_activate(Gtk::Widget&, const Glib::VariantBase&)
@@ -585,10 +586,10 @@ namespace gnote {
     }
 
     NoteManagerBase & manager(m_note.manager());
-    NoteBase::Ptr match = manager.find(title);
+    auto match = manager.find(title);
     if(!match) {
       try {
-        match = manager.create(std::move(select));
+        match = NoteBase::Ref(std::ref(manager.create(std::move(select))));
       }
       catch(const sharp::Exception & e) {
         auto dialog = Gtk::make_managed<utils::HIGMessageDialog>(dynamic_cast<Gtk::Window*>(m_note.get_window()->host()),
@@ -606,8 +607,7 @@ namespace gnote {
       buffer->apply_tag(m_note.get_tag_table()->get_link_tag(), start, end);
     }
 
-    MainWindow::present_in(*dynamic_cast<MainWindow*>(m_note.get_window()->host()),
-                           std::static_pointer_cast<Note>(match));
+    MainWindow::present_in(*dynamic_cast<MainWindow*>(m_note.get_window()->host()), static_cast<Note&>(match.value().get()));
   }
 
   //
@@ -954,7 +954,8 @@ namespace gnote {
       highlight->set_action_name("win.change-font-highlight");
       highlight->set_has_frame(false);
       auto label = Gtk::make_managed<Gtk::Label>();
-      Glib::ustring markup = Glib::ustring::compose("<span background=\"%1\">%2</span>", NoteTagTable::HIGHLIGHT_COLOR, _("_Highlight"));
+      Glib::ustring markup = Glib::ustring::compose("<span color=\"%1\" background=\"%2\">%3</span>",
+        NoteTagTable::HIGHLIGHT_TEXT_COLOR, NoteTagTable::HIGHLIGHT_COLOR, _("_Highlight"));
       label->set_markup_with_mnemonic(markup);
       highlight->set_child(*label);
 
