@@ -23,7 +23,6 @@
 
 #include "sharp/datetime.hpp"
 #include "debug.hpp"
-#include "notemanager.hpp"
 #include "noteoftheday.hpp"
 #include "utils.hpp"
 
@@ -34,13 +33,12 @@ const Glib::ustring NoteOfTheDay::s_template_title
 const Glib::ustring NoteOfTheDay::s_title_prefix
                                   = _("Today: ");
 
-gnote::NoteBase::Ptr NoteOfTheDay::create(gnote::NoteManager & manager,
-                                      const Glib::Date & date)
+gnote::NoteBase::ORef NoteOfTheDay::create(gnote::NoteManagerBase & manager, const Glib::Date & date)
 {
   Glib::ustring title = get_title(date);
   Glib::ustring xml = get_content(date, manager);
 
-  gnote::NoteBase::Ptr notd;
+  gnote::NoteBase::ORef notd;
   try {
     notd = manager.create(Glib::ustring(title), std::move(xml));
   }
@@ -49,26 +47,25 @@ gnote::NoteBase::Ptr NoteOfTheDay::create(gnote::NoteManager & manager,
     ERR_OUT(_("NoteOfTheDay could not create %s: %s"),
             title.c_str(),
             e.what());
-    return gnote::NoteBase::Ptr();
+    return gnote::NoteBase::ORef();
   }
 
   // Automatically tag all new Note of the Day notes
-  notd->add_tag(manager.tag_manager().get_or_create_system_tag("NoteOfTheDay"));
+  notd.value().get().add_tag(manager.tag_manager().get_or_create_system_tag("NoteOfTheDay"));
 
   return notd;
 }
 
-void NoteOfTheDay::cleanup_old(gnote::NoteManager & manager)
+void NoteOfTheDay::cleanup_old(gnote::NoteManagerBase & manager)
 {
-  gnote::NoteBase::List kill_list;
-  const gnote::NoteBase::List & notes = manager.get_notes();
+  std::vector<gnote::NoteBase::Ref> kill_list;
 
   Glib::Date date;
   date.set_time_current(); // time set to 00:00:00
 
-  for(const gnote::NoteBase::Ptr & note : notes) {
-    const Glib::ustring & title = note->get_title();
-    const auto & date_time = note->create_date();
+  manager.for_each([&kill_list, date](gnote::NoteBase & note) {
+    const Glib::ustring & title = note.get_title();
+    const auto & date_time = note.create_date();
 
     if (true == Glib::str_has_prefix(title, s_title_prefix)
         && s_template_title != title
@@ -79,23 +76,23 @@ void NoteOfTheDay::cleanup_old(gnote::NoteManager & manager)
         && !has_changed(note)) {
       kill_list.push_back(note);
     }
-  }
+  });
 
-  for(gnote::NoteBase::Ptr & note : kill_list) {
-    DBG_OUT("NoteOfTheDay: Deleting old unmodified '%s'", note->get_title().c_str());
-    manager.delete_note(*note);
+  for(const auto & note : kill_list) {
+    DBG_OUT("NoteOfTheDay: Deleting old unmodified '%s'", note.get().get_title().c_str());
+    manager.delete_note(note);
   }
 }
 
-Glib::ustring NoteOfTheDay::get_content(const Glib::Date & date, const gnote::NoteManager & manager)
+Glib::ustring NoteOfTheDay::get_content(const Glib::Date & date, const gnote::NoteManagerBase & manager)
 {
   const Glib::ustring title = get_title(date);
 
   // Attempt to load content from template
-  const gnote::NoteBase::Ptr template_note = manager.find(s_template_title);
+  auto template_note = manager.find(s_template_title);
 
-  if (0 != template_note) {
-    Glib::ustring xml_content = template_note->xml_content();
+  if(template_note) {
+    Glib::ustring xml_content = template_note.value().get().xml_content();
     return xml_content.replace(xml_content.find(s_template_title, 0),
                                s_template_title.length(),
                                title);
@@ -115,15 +112,11 @@ Glib::ustring NoteOfTheDay::get_content_without_title(const Glib::ustring & cont
     return Glib::ustring();
 }
 
-gnote::NoteBase::Ptr NoteOfTheDay::get_note_by_date(
-                                 gnote::NoteManager & manager,
-                                 const Glib::Date & date)
+gnote::NoteBase::ORef NoteOfTheDay::get_note_by_date(gnote::NoteManagerBase & manager, const Glib::Date & date)
 {
-  const gnote::NoteBase::List & notes = manager.get_notes();
-
-  for(gnote::NoteBase::Ptr note : notes) {
-    const Glib::ustring & title = note->get_title();
-    const auto & date_time = note->create_date();
+  return manager.search([date](gnote::NoteBase & note, gnote::NoteBase::ORef & ret) {
+    const Glib::ustring & title = note.get_title();
+    const auto & date_time = note.create_date();
 
     if (true == Glib::str_has_prefix(title, s_title_prefix)
         && s_template_title != title
@@ -131,11 +124,11 @@ gnote::NoteBase::Ptr NoteOfTheDay::get_note_by_date(
              date_time.get_day_of_month(),
              static_cast<Glib::Date::Month>(date_time.get_month()),
              date_time.get_year()) == date) {
-      return note;
+      ret = note;
+      return false;
     }
-  }
-
-  return gnote::Note::Ptr();
+    return true;
+  }, gnote::NoteBase::ORef());
 }
 
 Glib::ustring NoteOfTheDay::get_template_content(const Glib::ustring & title)
@@ -157,17 +150,17 @@ Glib::ustring NoteOfTheDay::get_title(const Glib::Date & date)
   return s_title_prefix + date.format_string(_("%A, %B %d %Y"));
 }
 
-bool NoteOfTheDay::has_changed(const gnote::NoteBase::Ptr & note)
+bool NoteOfTheDay::has_changed(gnote::NoteBase & note)
 {
-  const auto & date_time = note->create_date();
+  const auto & date_time = note.create_date();
   const Glib::ustring original_xml
     = get_content(Glib::Date(
                     date_time.get_day_of_month(),
                     static_cast<Glib::Date::Month>(date_time.get_month()),
                     date_time.get_year()),
-                    *static_cast<gnote::NoteManager*>(&note->manager()));
+                    note.manager());
 
-  return get_content_without_title(note->text_content()) != get_content_without_title(gnote::utils::XmlDecoder::decode(original_xml));
+  return get_content_without_title(note.text_content()) != get_content_without_title(gnote::utils::XmlDecoder::decode(original_xml));
 }
 
 }
