@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2015,2017,2019-2023 Aurimas Cernius
+ * Copyright (C) 2010-2015,2017,2019-2024 Aurimas Cernius
  * Copyright (C) 2009 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -45,7 +45,6 @@
 #include "notewindow.hpp"
 #include "preferencesdialog.hpp"
 #include "preferences.hpp"
-#include "preferencetabaddin.hpp"
 #include "utils.hpp"
 #include "watchers.hpp"
 
@@ -102,6 +101,49 @@ namespace {
     AddinInfo m_addin_info;
     Glib::ustring m_id;
   };
+
+
+  class ColorSchemeItem
+    : public Glib::Object
+  {
+  public:
+    static Glib::RefPtr<ColorSchemeItem> create(Glib::ustring &&label, Glib::ustring &&value)
+      {
+        return Glib::make_refptr_for_instance(new ColorSchemeItem(std::move(label), std::move(value)));
+      }
+
+    const Glib::ustring label;
+    const Glib::ustring value;
+  private:
+    ColorSchemeItem(Glib::ustring &&label, Glib::ustring &&value)
+      : label(std::move(label))
+      , value(std::move(value))
+      {}
+  };
+
+  Glib::RefPtr<Gtk::Expression<Glib::ustring>> make_color_scheme_label_expr()
+  {
+    return Gtk::ClosureExpression<Glib::ustring>::create([](const Glib::RefPtr<Glib::ObjectBase> &obj) -> Glib::ustring {
+      if(auto item = std::dynamic_pointer_cast<ColorSchemeItem>(obj)) {
+        return item->label;
+      }
+
+      ERR_OUT("Object is not ColorSchemeItem. This is a bug, please report it!");
+      return "";
+    });
+  }
+
+  guint color_scheme_item_idx(const Glib::RefPtr<Gio::ListStore<ColorSchemeItem>> &model, const Glib::ustring &value)
+  {
+    const auto count = model->get_n_items();
+    for(guint i = 0; i < count; ++i) {
+      if(value == model->get_item(i)->value) {
+        return i;
+      }
+    }
+
+    return 0;
+  }
 }
 
 
@@ -126,31 +168,12 @@ namespace {
 
     Gtk::Notebook *notebook = Gtk::make_managed<Gtk::Notebook>();
     notebook->set_margin(5);
-    
+
     notebook->append_page(*make_editing_pane(), _("General"));
     notebook->append_page(*make_links_pane(), _("Links"));
     notebook->append_page(*make_sync_pane(), _("Synchronization"));
     notebook->append_page(*make_addins_pane(), _("Plugins"));
 
-      // TODO: Figure out a way to have these be placed in a specific order
-    std::vector<PreferenceTabAddin*> tabAddins = m_addin_manager.get_preference_tab_addins();
-    for(auto tabAddin : tabAddins) {
-      DBG_OUT("Adding preference tab addin: %s", 
-              typeid(*tabAddin).name());
-        try {
-          Glib::ustring tabName;
-          Gtk::Widget *tabWidget = NULL;
-          if (tabAddin->get_preference_tab_widget (this, tabName, tabWidget)) {
-            notebook->append_page (*manage(tabWidget), tabName);
-          }
-        } 
-        catch(...)
-        {
-          DBG_OUT("Problems adding preferences tab addin: %s", 
-                  typeid(*tabAddin).name());
-        }
-      }
-
     get_content_area()->append(*notebook);
 
     Gtk::Button *button = Gtk::make_managed<Gtk::Button>(_("_Close"), true);
@@ -261,27 +284,23 @@ namespace {
       bullet_peditor->setup();
 
       // Custom font...
-
-      auto font_box = Gtk::make_managed<Gtk::Grid>();
       check = make_check_button(_("Use custom _font"));
       check->set_hexpand(true);
-      font_box->attach(*check, 0, 0, 1, 1);
+      options_list->attach(*check, 0, options_list_row, 1, 1);
       font_peditor = NEW_PROPERTY_EDITOR_BOOL(enable_custom_font, *check);
       font_peditor->setup();
 
       font_button = manage(make_font_button());
       font_button->set_sensitive(check->get_active());
       font_button->set_hexpand(true);
-      font_box->attach(*font_button, 1, 0, 1, 1);
-      options_list->attach(*font_box, 0, options_list_row++, 1, 1);
+      options_list->attach(*font_button, 1, options_list_row++, 1, 1);
 
       font_peditor->add_guard(font_button);
 
       // Note renaming behavior
-      auto rename_behavior_box = Gtk::make_managed<Gtk::Grid>();
       label = make_label(_("When renaming a linked note: "));
       label->set_hexpand(true);
-      rename_behavior_box->attach(*label, 0, 0, 1, 1);
+      options_list->attach(*label, 0, options_list_row, 1, 1);
       std::vector<Glib::ustring> rename_opts;
       rename_opts.emplace_back(_("Ask me what to do"));
       rename_opts.emplace_back(_("Never rename links"));
@@ -293,34 +312,41 @@ namespace {
         m_gnote.preferences().note_rename_behavior(rename_behavior);
       }
       m_rename_behavior_combo->set_selected(rename_behavior);
-      m_rename_behavior_combo->signal_state_flags_changed().connect([this](Gtk::StateFlags prev_flags) {
-        auto was_active = (Gtk::StateFlags::ACTIVE == (prev_flags & Gtk::StateFlags::ACTIVE));
-        auto now_active = (Gtk::StateFlags::ACTIVE == (m_rename_behavior_combo->get_state_flags() & Gtk::StateFlags::ACTIVE));
-        if(!now_active && was_active) {
-          auto on_deactivate = [](gpointer data) {
-            static_cast<PreferencesDialog*>(data)->on_rename_behavior_changed();
-          };
-          g_timeout_add_once(1, on_deactivate, this);
+      m_rename_behavior_combo->property_selected().signal_changed().connect(sigc::mem_fun(*this, &PreferencesDialog::on_rename_behavior_changed));
+      m_rename_behavior_combo->set_hexpand(true);
+      options_list->attach(*m_rename_behavior_combo, 1, options_list_row++, 1, 1);
+
+      // Color scheme
+      label = make_label(_("Color Scheme: "));
+      options_list->attach(*label, 0, options_list_row, 1, 1);
+      auto color_scheme_model = Gio::ListStore<ColorSchemeItem>::create();
+      // TRANSLATORS: Option to follow the system color scheme setting
+      color_scheme_model->append(ColorSchemeItem::create(_("Follow the System"), ""));
+      // TRANSLATORS: Option to use dark variant of the theme
+      color_scheme_model->append(ColorSchemeItem::create(_("Dark"), Preferences::COLOR_SCHEME_DARK_VAL));
+      // TRANSLATORS: Option to use light variant of the theme
+      color_scheme_model->append(ColorSchemeItem::create(_("Light"), Preferences::COLOR_SCHEME_LIGHT_VAL));
+      auto color_scheme = Gtk::make_managed<Gtk::DropDown>(color_scheme_model, make_color_scheme_label_expr());
+      color_scheme->set_selected(color_scheme_item_idx(color_scheme_model, m_gnote.preferences().color_scheme()));
+      color_scheme->property_selected().signal_changed().connect([color_scheme, &gnote=m_gnote] {
+        if(auto item = std::dynamic_pointer_cast<ColorSchemeItem>(color_scheme->get_selected_item())) {
+          gnote.preferences().color_scheme(item->value);
         }
       });
-      m_rename_behavior_combo->set_hexpand(true);
-      rename_behavior_box->attach(*m_rename_behavior_combo, 1, 0, 1, 1);
-      options_list->attach(*rename_behavior_box, 0, options_list_row++, 1, 1);
+      options_list->attach(*color_scheme, 1, options_list_row++, 1, 1);
 
       // New Note Template
-      auto template_note_grid = Gtk::make_managed<Gtk::Grid>();
       // TRANSLATORS: This is 'New Note' Template, not New 'Note Template'
       label = make_label(_("New Note Template"));
       set_widget_tooltip(*label, _("Use the new note template to specify the text "
                                    "that should be used when creating a new note."));
       label->set_hexpand(true);
-      template_note_grid->attach(*label, 0, 0, 1, 1);
+      options_list->attach(*label, 0, options_list_row, 1, 1);
       
       Gtk::Button *open_template_button = Gtk::make_managed<Gtk::Button>(_("Open New Note Template"));
       open_template_button->signal_clicked().connect(
         sigc::mem_fun(*this, &PreferencesDialog::open_template_button_clicked));
-      template_note_grid->attach(*open_template_button, 1, 0, 1, 1);
-      options_list->attach(*template_note_grid, 0, options_list_row++, 1, 1);
+      options_list->attach(*open_template_button, 1, options_list_row++, 1, 1);
 
       return options_list;
     }
@@ -876,12 +902,10 @@ namespace {
 
 
 
-  void  PreferencesDialog::open_template_button_clicked()
+  void PreferencesDialog::open_template_button_clicked()
   {
-    NoteBase::Ptr template_note = m_note_manager.get_or_create_template_note();
-
     // Open the template note
-    m_gnote.open_note(*template_note);
+    m_gnote.open_note(m_note_manager.get_or_create_template_note());
   }
 
 
