@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2023 Aurimas Cernius
+ * Copyright (C) 2010-2024 Aurimas Cernius
  * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -45,6 +45,8 @@ namespace gnote {
   namespace {
     const char *MAIN_MENU_PRIMARY_ICON = "open-menu-symbolic";
     const char *MAIN_MENU_SECONDARY_ICON = "view-more-symbolic";
+    const char *BUTTON_FIND_NEXT = "find-next";
+    const char *BUTTON_FIND_PREV = "find-prev";
 
     class TabLabel
       : public Gtk::Box
@@ -99,7 +101,6 @@ namespace gnote {
     , m_note_manager(m)
     , m_preferences(g.preferences())
     , m_search_box(nullptr)
-    , m_find_next_prev_box(nullptr)
     , m_search_entry(nullptr)
     , m_mapped(false)
   {
@@ -340,10 +341,10 @@ namespace gnote {
     }
   }
 
-  void NoteRecentChanges::make_search_box()
+  bool NoteRecentChanges::make_search_box()
   {
     if(m_search_box) {
-      return;
+      return false;
     }
 
     Glib::ustring search_text;
@@ -366,50 +367,87 @@ namespace gnote {
     m_search_entry->signal_previous_match()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_find_prev_button_clicked));
 
-    m_search_box = manage(new Gtk::Grid);
+    m_search_box = Gtk::make_managed<Gtk::Box>();
+    m_search_box->add_css_class("linked");
     m_search_box->set_hexpand(false);
-    m_search_box->attach(*m_search_entry, 0, 0, 1, 1);
+    m_search_box->append(*m_search_entry);
     m_search_box->set_halign(Gtk::Align::CENTER);
 
     auto content = dynamic_cast<Gtk::Grid*>(m_search_button.get_parent());
     if(content) {
       content->attach_next_to(*m_search_box, m_search_button, Gtk::PositionType::LEFT);
+      return true;
     }
     else {
       ERR_OUT(_("Parent of embed box is not a Gtk::Grid, please report a bug!"));
     }
+
+    return false;
   }
 
-  void NoteRecentChanges::make_find_next_prev()
+  void NoteRecentChanges::make_find_next_prev(Gtk::Button *&find_next_button, Gtk::Button *&find_prev_button)
   {
-    if(m_find_next_prev_box) {
-      return;
-    }
-
-    m_find_next_prev_box = manage(new Gtk::Grid);
-    m_find_next_prev_box->set_margin_start(5);
-
-    Gtk::Button *find_next_button = manage(new Gtk::Button);
+    find_next_button = Gtk::make_managed<Gtk::Button>();
+    find_next_button->set_name(BUTTON_FIND_NEXT);
     find_next_button->property_icon_name() = "go-down-symbolic";
     find_next_button->signal_clicked()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_find_next_button_clicked));
-    m_find_next_prev_box->attach(*find_next_button, 0, 0, 1, 1);
 
-    Gtk::Button *find_prev_button = manage(new Gtk::Button);
+    find_prev_button = Gtk::make_managed<Gtk::Button>();
+    find_prev_button->set_name(BUTTON_FIND_PREV);
     find_prev_button->property_icon_name() = "go-up-symbolic";
     find_prev_button->signal_clicked()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_find_prev_button_clicked));
-    m_find_next_prev_box->attach(*find_prev_button, 1, 0, 1, 1);
 
-    auto grid = dynamic_cast<Gtk::Grid*>(m_search_entry->get_parent());
-    if(grid) {
-      grid->attach(*m_find_next_prev_box, 1, 0, 1, 1);
+    if(auto box = dynamic_cast<Gtk::Box*>(m_search_entry->get_parent())) {
+      box->append(*find_next_button);
+      box->append(*find_prev_button);
     }
     else {
       ERR_OUT(_("Parent of search entry is not Gtk::Grid, please report a bug!"));
     }
   }
 
+  void NoteRecentChanges::show_find_next_prev()
+  {
+    Gtk::Button *find_next, *find_prev;
+    if(!get_find_next_prev(find_next, find_prev)) {
+      make_find_next_prev(find_next, find_prev);
+    }
+
+    find_next->show();
+    find_prev->show();
+  }
+
+  void NoteRecentChanges::hide_find_next_prev()
+  {
+    Gtk::Button *find_next, *find_prev;
+    if(get_find_next_prev(find_next, find_prev)) {
+      find_next->hide();
+      find_prev->hide();
+    }
+  }
+
+  bool NoteRecentChanges::get_find_next_prev(Gtk::Button *&find_next, Gtk::Button *&find_prev) const
+  {
+    find_next = find_prev = nullptr;
+    if(m_search_entry == nullptr) {
+      return false;
+    }
+
+    Gtk::Widget *widget = m_search_entry;
+    while((widget = widget->get_next_sibling())) {
+      if(widget->get_name() == BUTTON_FIND_NEXT) {
+        find_next = dynamic_cast<Gtk::Button*>(widget);
+      }
+      else if(widget->get_name() == BUTTON_FIND_PREV) {
+        find_prev = dynamic_cast<Gtk::Button*>(widget);
+      }
+    }
+
+    return find_next != nullptr && find_prev != nullptr;
+  }
+
   void NoteRecentChanges::on_search_button_toggled()
   {
     if(m_search_button.get_active()) {
@@ -442,8 +480,8 @@ namespace gnote {
 
   void NoteRecentChanges::show_search_bar(bool focus)
   {
-    make_search_box();
-    if(m_search_box->get_visible()) {
+    if(!make_search_box() && m_search_box->get_visible()) {
+      // grab focus only if freshly create or became visible
       focus = false;
     }
     m_search_box->show();
@@ -461,14 +499,11 @@ namespace gnote {
       m_search_button.show();
       if(searchable_item->supports_goto_result()) {
         if(m_search_box && m_search_box->get_visible()) {
-          make_find_next_prev();
-          m_find_next_prev_box->show();
+          show_find_next_prev();
         }
       }
       else {
-        if(m_find_next_prev_box) {
-          m_find_next_prev_box->hide();
-        }
+        hide_find_next_prev();
       }
       if(perform_search) {
         searchable_item->perform_search(m_search_button.get_active() ? m_search_entry->get_text() : "");
@@ -492,16 +527,13 @@ namespace gnote {
     foreground_embedded(*m_search_notes_widget);
   }
 
-  void NoteRecentChanges::present_note(const Note::Ptr & note)
+  void NoteRecentChanges::present_note(Note & note)
   {
-    if(!note) {
-      return;
-    }
     if(present_active(note)) {
       return;
     }
-    if(note->has_window()) {
-      auto win = note->get_window();
+    if(note.has_window()) {
+      auto win = note.get_window();
       if(win->host()) {
         win->host()->unembed_widget(*win);
       }
@@ -509,7 +541,7 @@ namespace gnote {
       win->set_initial_focus();
     }
     else {
-      auto win = note->create_window();
+      auto win = note.create_window();
       embed_widget(*win);
       win->set_initial_focus();
     }
@@ -524,20 +556,20 @@ namespace gnote {
       search_wgt->new_note();
     }
     else {
-      present_note(std::static_pointer_cast<Note>(m_note_manager.create()));
+      present_note(static_cast<Note&>(m_note_manager.create()));
     }
   }
 
 
-  void NoteRecentChanges::on_open_note_new_window(const Note::Ptr & note)
+  void NoteRecentChanges::on_open_note_new_window(Note & note)
   {
     present_in_new_window(m_gnote, note);
   }
 
-  bool NoteRecentChanges::present_active(const Note::Ptr & note)
+  bool NoteRecentChanges::present_active(Note & note)
   {
-    if(note && note->has_window()) {
-      auto note_window = note->get_window();
+    if(note.has_window()) {
+      auto note_window = note.get_window();
       auto win = dynamic_cast<NoteRecentChanges*>(note_window->host());
       if(!win) {
         return false;
