@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2011,2013-2014,2017,2019 Aurimas Cernius
+ * Copyright (C) 2011,2013-2014,2017,2019,2023 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -35,8 +35,8 @@ namespace gnote {
   }
 
 
-  Search::ResultsPtr Search::search_notes(const Glib::ustring & query, bool case_sensitive,
-                                  const notebooks::Notebook::Ptr & selected_notebook)
+  Search::Results Search::search_notes(const Glib::ustring & query, bool case_sensitive,
+                                       notebooks::Notebook::ORef selected_notebook)
   {
     Glib::ustring search_text = query;
     if(!case_sensitive) {
@@ -49,47 +49,47 @@ namespace gnote {
     // Used for matching in the raw note XML
     std::vector<Glib::ustring> encoded_words;
     Search::split_watching_quotes(encoded_words, utils::XmlEncoder::encode(search_text));
-    ResultsPtr temp_matches(std::make_shared<Results>());
+    Results temp_matches;
       
       // Skip over notes that are template notes
     Tag::Ptr template_tag = m_manager.tag_manager().get_or_create_system_tag(ITagManager::TEMPLATE_NOTE_SYSTEM_TAG);
 
-    for(const NoteBase::Ptr & iter : m_manager.get_notes()) {
-      Note::Ptr note(std::static_pointer_cast<Note>(iter));
-
+    m_manager.for_each([this, &temp_matches, template_tag, selected_notebook, case_sensitive, words=std::move(words), encoded_words=std::move(encoded_words)](NoteBase & note) {
       // Skip template notes
-      if (note->contains_tag (template_tag)) {
-        continue;
+      if(note.contains_tag(template_tag)) {
+        return;
       }
         
       // Skip notes that are not in the
       // selected notebook
-      if (selected_notebook && !selected_notebook->contains_note(note))
-        continue;
+      if(selected_notebook && !selected_notebook.value().get().contains_note(static_cast<Note&>(note))) {
+        return;
+      }
         
       // First check the note's title for a match,
       // if there is no match check the note's raw
       // XML for at least one match, to avoid
       // deserializing Buffers unnecessarily.
-      if (0 < find_match_count_in_note(note->get_title(), words, case_sensitive)) {
-        temp_matches->insert(std::make_pair(INT_MAX, note));
+      if(0 < find_match_count_in_note(note.get_title(), words, case_sensitive)) {
+        temp_matches.insert(std::make_pair(INT_MAX, std::ref(note)));
       }
-      else if (check_note_has_match(note, encoded_words, case_sensitive)) {
-        int match_count = find_match_count_in_note(note->text_content(), words, case_sensitive);
+      else if(check_note_has_match(note, encoded_words, case_sensitive)) {
+        int match_count = find_match_count_in_note(note.text_content(), words, case_sensitive);
         if (match_count > 0) {
           // TODO: Improve note.GetHashCode()
-          temp_matches->insert(std::make_pair(match_count, note));
+          temp_matches.insert(std::make_pair(match_count, std::ref(note)));
         }
       }
-    }
+    });
+
     return temp_matches;
   }
 
-  bool Search::check_note_has_match(const Note::Ptr & note, 
+  bool Search::check_note_has_match(const NoteBase & note,
                                     const std::vector<Glib::ustring> & encoded_words,
                                     bool match_case)
   {
-    Glib::ustring note_text = note->xml_content();
+    Glib::ustring note_text = note.xml_content();
     if (!match_case) {
       note_text = note_text.lowercase();
     }
