@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2012-2014,2016,2017,2019-2023 Aurimas Cernius
+ * Copyright (C) 2012-2014,2016,2017,2019-2024 Aurimas Cernius
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,10 +60,6 @@ protected:
     return get_text(*record);
   }
   virtual Glib::ustring get_text(ListRecord & record) = 0;
-  void set_text(Gtk::Label & label, const Glib::ustring & text) override
-  {
-    label.set_text(text);
-  }
 };
 
 
@@ -93,14 +89,14 @@ class SyncTitleConflictDialog
   : public Gtk::Dialog
 {
 public:
-  SyncTitleConflictDialog(const Note::Ptr & existingNote, const std::vector<Glib::ustring> & noteUpdateTitles)
+  SyncTitleConflictDialog(Note & existingNote, const std::vector<Glib::ustring> & noteUpdateTitles)
     : Gtk::Dialog(_("Note Conflict"), true)
-    , m_existing_note(existingNote)
+    , m_note_manager(existingNote.manager())
     , m_note_update_titles(noteUpdateTitles)
     {
       // Suggest renaming note by appending " (old)" to the existing title
       char *old = _(" (old)");
-      Glib::ustring suggestedRenameBase = existingNote->get_title() + old;
+      Glib::ustring suggestedRenameBase = existingNote.get_title() + old;
       Glib::ustring suggestedRename = suggestedRenameBase;
       for(int i = 1; !is_note_title_available(suggestedRename); i++) {
         suggestedRename = suggestedRenameBase + " " + TO_STRING(i);
@@ -166,7 +162,7 @@ public:
       header_text(_("Note conflict detected"));
       message_text(Glib::ustring::compose(
         _("The server version of \"%1\" conflicts with your local note.  What do you want to do with your local note?"),
-        existingNote->get_title()));
+        existingNote.get_title()));
     }
   void header_text(const Glib::ustring & value)
     {
@@ -212,7 +208,7 @@ private:
   bool is_note_title_available(const Glib::ustring & renamedTitle)
     {
       return std::find(m_note_update_titles.begin(), m_note_update_titles.end(), renamedTitle) == m_note_update_titles.end()
-             && m_existing_note->manager().find(renamedTitle) == 0;
+             && m_note_manager.find(renamedTitle);
     }
   void radio_toggled()
     {
@@ -224,7 +220,7 @@ private:
       renameUpdateCheck->set_sensitive(renameRadio->get_active());
     }
 
-  Note::Ptr m_existing_note;
+  NoteManagerBase & m_note_manager;
   std::vector<Glib::ustring> m_note_update_titles;
 
   Gtk::Button *continueButton;
@@ -406,15 +402,16 @@ void SyncDialog::on_row_activated(guint idx)
 
   Glib::ustring noteTitle = item->value.title;
 
-  NoteBase::Ptr note = m_manager.find(noteTitle);
-  if(note != 0) {
-    present_note(std::static_pointer_cast<Note>(note));
+  if(auto note = m_manager.find(noteTitle)) {
+    present_note(static_cast<Note&>(note.value().get()));
   }
 }
 
 
 void SyncDialog::present_ui()
 {
+  auto &window = m_gnote.get_window_for_note();
+  set_transient_for(window);
   present();
 }
 
@@ -576,7 +573,7 @@ void SyncDialog::note_synchronized(const Glib::ustring & noteTitle, NoteSyncType
 }
 
 
-void SyncDialog::note_conflict_detected(const Note::Ptr & localConflictNote,
+void SyncDialog::note_conflict_detected(NoteBase & localConflictNote,
                                         NoteUpdate remoteNote,
                                         const std::vector<Glib::ustring> & noteUpdateTitles)
 {
@@ -590,12 +587,20 @@ void SyncDialog::note_conflict_detected(const Note::Ptr & localConflictNote,
   // so we have to use the delegate here to manipulate the GUI.
   // To be consistent, any exceptions in the delgate will be caught
   // and then rethrown in the synchronization thread.
+  auto local_conflict_note = localConflictNote.uri();
+  auto & manager = localConflictNote.manager();
   utils::main_context_invoke(
-    [this, localConflictNote, remoteNote, noteUpdateTitles, dlgBehaviorPref, &wait_mutex, &wait, &completed]() {
-      note_conflict_detected_(localConflictNote, remoteNote, noteUpdateTitles,
-                              static_cast<SyncTitleConflictResolution>(dlgBehaviorPref),
-                              OVERWRITE_EXISTING, wait_mutex, wait, completed
-      );
+    [this, &manager, local_conflict_note, remoteNote, noteUpdateTitles, dlgBehaviorPref, &wait_mutex, &wait, &completed]() {
+      if(auto note = manager.find_by_uri(local_conflict_note)) {
+        note_conflict_detected_(static_cast<Note&>(note.value().get()), remoteNote, noteUpdateTitles,
+                                static_cast<SyncTitleConflictResolution>(dlgBehaviorPref),
+                                OVERWRITE_EXISTING, wait_mutex, wait, completed
+        );
+      }
+      else {
+        std::unique_lock lock(wait_mutex);
+        wait.notify_one();
+      }
     });
 
   while(!completed) {
@@ -605,7 +610,7 @@ void SyncDialog::note_conflict_detected(const Note::Ptr & localConflictNote,
 
 
 void SyncDialog::note_conflict_detected_(
-  const Note::Ptr & localConflictNote,
+  Note & localConflictNote,
   NoteUpdate remoteNote,
   const std::vector<Glib::ustring> & noteUpdateTitles,
   SyncTitleConflictResolution savedBehavior,
@@ -615,18 +620,19 @@ void SyncDialog::note_conflict_detected_(
   bool & completed)
 {
   bool noteSyncBitsMatch = m_gnote.sync_manager().synchronized_note_xml_matches(
-    localConflictNote->get_complete_note_xml(), remoteNote.m_xml_content);
+    localConflictNote.get_complete_note_xml(), remoteNote.m_xml_content);
 
   // If the synchronized note content is in conflict
   // and there is no saved conflict handling behavior, show the dialog
   if(!noteSyncBitsMatch && savedBehavior == 0) {
     auto conflictDlg = Gtk::make_managed<SyncTitleConflictDialog>(localConflictNote, noteUpdateTitles);
+    auto local_conflict_note = localConflictNote.uri();
     conflictDlg->signal_response()
-      .connect([this, conflictDlg, localConflictNote, remoteNote, savedBehavior, resolution, noteSyncBitsMatch, &wait_mutex, &wait, &completed](int resp) {
+      .connect([this, conflictDlg, local_conflict_note, remoteNote, savedBehavior, resolution, noteSyncBitsMatch, &wait_mutex, &wait, &completed](int resp) {
       auto response = static_cast<Gtk::ResponseType>(resp);
       conflict_dialog_response(
         conflictDlg,
-        localConflictNote,
+        local_conflict_note,
         remoteNote,
         savedBehavior,
         resolution,
@@ -650,7 +656,7 @@ void SyncDialog::note_conflict_detected_(
 
 void SyncDialog::conflict_dialog_response(
   Gtk::Dialog *dialog,
-  const Note::Ptr & localConflictNote,
+  const Glib::ustring & localConflictNote,
   NoteUpdate remoteNote,
   SyncTitleConflictResolution savedBehavior,
   SyncTitleConflictResolution resolution,
@@ -679,8 +685,10 @@ void SyncDialog::conflict_dialog_response(
         savedBehavior = resolution;
       }
       // No need to delete if sync will overwrite
-      if(localConflictNote->id() != remoteNote.m_uuid) {
-        m_manager.delete_note(*localConflictNote);
+      if(localConflictNote != remoteNote.m_uuid) {
+        if(auto local_conflict_note = m_manager.find_by_uri(localConflictNote)) {
+          m_manager.delete_note(local_conflict_note.value());
+        }
       }
       break;
     case RENAME_EXISTING_AND_UPDATE:
@@ -709,9 +717,15 @@ void SyncDialog::conflict_dialog_response(
 }
 
 
-void SyncDialog::rename_note(const Note::Ptr & note, Glib::ustring && newTitle, bool)
+void SyncDialog::rename_note(const Glib::ustring & note_uri, Glib::ustring && newTitle, bool)
 {
-  Glib::ustring oldTitle = note->get_title();
+  auto note_ref = m_manager.find_by_uri(note_uri);
+  if(!note_ref) {
+    return;
+  }
+
+  Note & note = static_cast<Note&>(note_ref.value().get());
+  Glib::ustring oldTitle = note.get_title();
   // Rename the note (skip for now...never using updateReferencingNotes option)
   //if (updateReferencingNotes) // NOTE: This might never work, or lead to a ton of conflicts
   // note.Title = newTitle;
@@ -721,33 +735,33 @@ void SyncDialog::rename_note(const Note::Ptr & note, Glib::ustring && newTitle,
   //note.XmlContent = NoteArchiver.Instance.GetRenamedNoteXml (oldContent, oldTitle, newTitle);
 
   // Preserve note information
-  note->save(); // Write to file
-  bool noteOpen = note->is_opened();
+  note.save(); // Write to file
+  bool noteOpen = note.is_opened();
   Glib::ustring newContent = //note.XmlContent;
-    m_manager.note_archiver().get_renamed_note_xml(note->xml_content(), oldTitle, newTitle);
+    m_manager.note_archiver().get_renamed_note_xml(note.xml_content(), oldTitle, newTitle);
   Glib::ustring newCompleteContent = //note.GetCompleteNoteXml ();
-    m_manager.note_archiver().get_renamed_note_xml(note->get_complete_note_xml(), oldTitle, newTitle);
+    m_manager.note_archiver().get_renamed_note_xml(note.get_complete_note_xml(), oldTitle, newTitle);
   //Logger.Debug ("RenameNote: newContent: " + newContent);
   //Logger.Debug ("RenameNote: newCompleteContent: " + newCompleteContent);
 
   // We delete and recreate the note to simplify content conflict handling
-  m_manager.delete_note(*note);
+  m_manager.delete_note(note);
 
   // Create note with old XmlContent just in case GetCompleteNoteXml failed
   DBG_OUT("RenameNote: about to create %s", newTitle.c_str());
-  Note::Ptr renamedNote = std::static_pointer_cast<Note>(m_manager.create(std::move(newTitle), std::move(newContent)));
-  if(newCompleteContent != "") {// TODO: Anything to do if it is null?
+  auto & renamedNote = m_manager.create(std::move(newTitle), std::move(newContent));
+  if(newCompleteContent != "") {
     try {
-      renamedNote->load_foreign_note_xml(newCompleteContent, OTHER_DATA_CHANGED);
+      renamedNote.load_foreign_note_xml(newCompleteContent, OTHER_DATA_CHANGED);
     }
     catch(...) {} // TODO: Handle exception in case that newCompleteContent is invalid XML
   }
   if(noteOpen) {
-    present_note(renamedNote);
+    present_note(static_cast<Note&>(renamedNote));
   }
 }
 
-void SyncDialog::present_note(const Note::Ptr & note)
+void SyncDialog::present_note(Note & note)
 {
   MainWindow::present_in(m_gnote.get_window_for_note(), note);
 }
