@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2012-2014,2017,2019-2022 Aurimas Cernius
+ * Copyright (C) 2012-2014,2017,2019-2023 Aurimas Cernius
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 #include "config.h"
 
 #include <glibmm/i18n.h>
-#include <gtkmm/actiongroup.h>
 #include <sigc++/sigc++.h>
 
 #include "iactionmanager.hpp"
@@ -223,12 +222,12 @@ namespace sync {
       //       Want this stuff to happen all at once first, but
       //       maybe there's a way to store this info and pass it on?
       for(auto & iter : noteUpdates) {
-        if(find_note_by_uuid(iter.second.m_uuid) != 0) {
-          NoteBase::Ptr existingNote = note_mgr().find(iter.second.m_title);
-          if(existingNote != 0 && !iter.second.basically_equal_to(std::static_pointer_cast<Note>(existingNote))) {
+        if(find_note_by_uuid(iter.second.m_uuid)) {
+          auto existingNote = note_mgr().find(iter.second.m_title);
+          if(existingNote && !iter.second.basically_equal_to(existingNote.value())) {
             DBG_OUT("Sync: Early conflict detection for '%s'", iter.second.m_title.c_str());
             if(m_sync_ui != 0) {
-              m_sync_ui->note_conflict_detected(std::static_pointer_cast<Note>(existingNote), iter.second, noteUpdateTitles);
+              m_sync_ui->note_conflict_detected(existingNote.value(), iter.second, noteUpdateTitles);
             }
           }
         }
@@ -241,40 +240,44 @@ namespace sync {
 
       // Process updates from the server; the bread and butter of sync!
       for(auto & iter : noteUpdates) {
-        NoteBase::Ptr existingNote = find_note_by_uuid(iter.second.m_uuid);
+        auto existing_note = find_note_by_uuid(iter.second.m_uuid);
 
-        if(existingNote == 0) {
+        if(!existing_note) {
           // Actually, it's possible to have a conflict here
           // because of automatically-created notes like
           // template notes (if a note with a new tag syncs
           // before its associated template). So check by
           // title and delete if necessary.
-          existingNote = note_mgr().find(iter.second.m_title);
-          if(existingNote != 0) {
+          auto existingNote = note_mgr().find(iter.second.m_title);
+          if(existingNote) {
             DBG_OUT("SyncManager: Deleting auto-generated note: %s", iter.second.m_title.c_str());
-            delete_note_in_main_thread(std::static_pointer_cast<Note>(existingNote));
+            delete_note_in_main_thread(existingNote.value());
           }
           create_note_in_main_thread(iter.second);
         }
-        else if(existingNote->metadata_change_date() <= m_client->last_sync_date()
-                || iter.second.basically_equal_to(std::static_pointer_cast<Note>(existingNote))) {
-          // Existing note hasn't been modified since last sync; simply update it from server
-          update_note_in_main_thread(std::static_pointer_cast<Note>(existingNote), iter.second);
-        }
         else {
-          // Logger.Debug ("Sync: Late conflict detection for '{0}'", noteUpdate.Title);
-          DBG_OUT("SyncManager: Content conflict in note update for note '%s'", iter.second.m_title.c_str());
-          // Note already exists locally, but has been modified since last sync; prompt user
-          if(m_sync_ui != 0) {
-            m_sync_ui->note_conflict_detected(std::static_pointer_cast<Note>(existingNote), iter.second, noteUpdateTitles);
+          NoteBase & existing = existing_note.value();
+          if(existing.metadata_change_date() <= m_client->last_sync_date()
+                || iter.second.basically_equal_to(existing)) {
+            // Existing note hasn't been modified since last sync; simply update it from server
+            update_note_in_main_thread(existing, iter.second);
           }
+          else {
+            // Logger.Debug ("Sync: Late conflict detection for '{0}'", noteUpdate.Title);
+            DBG_OUT("SyncManager: Content conflict in note update for note '%s'", iter.second.m_title.c_str());
+            // Note already exists locally, but has been modified since last sync; prompt user
+            if(m_sync_ui != 0) {
+              m_sync_ui->note_conflict_detected(existing, iter.second, noteUpdateTitles);
+            }
 
-          // Note has been deleted or okay'd for overwrite
-          existingNote = find_note_by_uuid(iter.second.m_uuid);
-          if(existingNote == 0)
-            create_note_in_main_thread(iter.second);
-          else
-            update_note_in_main_thread(std::static_pointer_cast<Note>(existingNote), iter.second);
+            if(auto existingNote = find_note_by_uuid(iter.second.m_uuid)) {
+              update_note_in_main_thread(existingNote.value(), iter.second);
+            }
+            else {
+              // Note has been deleted or okay'd for overwrite
+              create_note_in_main_thread(iter.second);
+            }
+          }
         }
       }
 
@@ -289,39 +292,38 @@ namespace sync {
       set_state(PREPARE_UPLOAD);
       // Look through all the notes modified on the client
       // and upload new or modified ones to the server
-      std::vector<Note::Ptr> newOrModifiedNotes;
-      for(const NoteBase::Ptr & iter : note_mgr().get_notes()) {
-        Note::Ptr note = std::static_pointer_cast<Note>(iter);
+      std::vector<NoteBase::Ref> new_or_modified_notes;
+      note_mgr().for_each([this, &new_or_modified_notes](NoteBase & note) {
         if(m_client->get_revision(note) == -1) {
           // This is a new note that has never been synchronized to the server
           // TODO: *OR* this is a note that we lost revision info for!!!
           // TODO: Do the above NOW!!! (don't commit this dummy)
           note_save(note);
-          newOrModifiedNotes.push_back(note);
+          new_or_modified_notes.push_back(note);
           if(m_sync_ui != 0)
-            m_sync_ui->note_synchronized_th(note->get_title(), UPLOAD_NEW);
+            m_sync_ui->note_synchronized_th(note.get_title(), UPLOAD_NEW);
         }
         else if(m_client->get_revision(note) <= m_client->last_synchronized_revision()
-                && note->metadata_change_date() > m_client->last_sync_date()) {
+                && note.metadata_change_date() > m_client->last_sync_date()) {
           note_save(note);
-          newOrModifiedNotes.push_back(note);
+          new_or_modified_notes.push_back(note);
           if(m_sync_ui != 0) {
-            m_sync_ui->note_synchronized_th(note->get_title(), UPLOAD_MODIFIED);
+            m_sync_ui->note_synchronized_th(note.get_title(), UPLOAD_MODIFIED);
           }
         }
-      }
+      });
 
-      DBG_OUT("Sync: Uploading %d note updates", int(newOrModifiedNotes.size()));
-      if(newOrModifiedNotes.size() > 0) {
+      DBG_OUT("Sync: Uploading %zu note updates", new_or_modified_notes.size());
+      if(new_or_modified_notes.size() > 0) {
         set_state(UPLOADING);
-        server->upload_notes(newOrModifiedNotes); // TODO: Callbacks to update GUI as upload progresses
+        server->upload_notes(new_or_modified_notes); // TODO: Callbacks to update GUI as upload progresses
       }
 
       // Handle notes deleted on client
       std::vector<Glib::ustring> locallyDeletedUUIDs;
       auto all_note_uuids = server->get_all_note_uuids();
       for(auto & iter : all_note_uuids) {
-        if(find_note_by_uuid(iter) == 0) {
+        if(!find_note_by_uuid(iter)) {
           locallyDeletedUUIDs.push_back(iter);
           if(m_sync_ui != 0) {
             Glib::ustring deletedTitle = iter;
@@ -343,7 +345,7 @@ namespace sync {
       if(commitResult) {
         // Apply this revision number to all new/modified notes since last sync
         // TODO: Is this the best place to do this (after successful server commit)
-        for(auto & iter : newOrModifiedNotes) {
+        for(NoteBase & iter : new_or_modified_notes) {
           m_client->set_revision(iter, newRevision);
         }
         set_state(SUCCEEDED);
@@ -364,10 +366,6 @@ namespace sync {
       set_state(IDLE);
 
     }
-    catch(Glib::Exception & e) {
-      ERR_OUT(_("Synchronization failed with the following Glib exception: %s"), e.what().c_str());
-      abort_sync(server.get());
-    }
     catch(std::exception & e) { // top-level try
       ERR_OUT(_("Synchronization failed with the following exception: %s"), e.what());
       abort_sync(server.get());
@@ -403,7 +401,7 @@ namespace sync {
   }
 
 
-  void SyncManager::handle_note_buffer_changed(const NoteBase::Ptr &)
+  void SyncManager::handle_note_buffer_changed(NoteBase &)
   {
     // Note changed, iff a sync is coming up we kill the
     // timer to avoid interupting the user (we want to
@@ -439,7 +437,7 @@ namespace sync {
   }
 
 
-  void SyncManager::handle_note_saved_or_deleted(const NoteBase::Ptr &)
+  void SyncManager::handle_note_saved_or_deleted(NoteBase &)
   {
     if(m_sync_thread == NULL && m_autosync_timeout_pref_minutes > 0) {
       DBG_OUT("Note saved or deleted...restarting sync timer");
@@ -477,13 +475,13 @@ namespace sync {
       bool server_has_updates = false;
       bool client_has_updates = m_client->deleted_note_titles().size() > 0;
       if(!client_has_updates) {
-        for(const NoteBase::Ptr & iter : note_mgr().get_notes()) {
-          Note::Ptr note = std::static_pointer_cast<Note>(iter);
-          if(m_client->get_revision(note) == -1 || note->metadata_change_date() > m_client->last_sync_date()) {
-            client_has_updates = true;
-            break;
+        client_has_updates = note_mgr().search([this](const NoteBase & note, bool & has_updates) {
+          if(m_client->get_revision(note) == -1 || note.metadata_change_date() > m_client->last_sync_date()) {
+            has_updates = true;
+            return false;
           }
-        }
+          return true;
+        }, false);
       }
 
       // NOTE: Important to check, at least to verify
@@ -571,46 +569,57 @@ namespace sync {
   }
 
 
-  void SyncManager::update_note_in_main_thread(const Note::Ptr & existingNote, const NoteUpdate & noteUpdate)
+  void SyncManager::update_note_in_main_thread(const NoteBase & existing_note, const NoteUpdate & note_update)
   {
     // Note update may affect the GUI, so we have to use the
     // delegate to run in the main gtk thread.
-    // To be consistent, any exceptions in the delgate will be caught
-    // and then rethrown in the synchronization thread.
-    utils::main_context_call([this, existingNote, noteUpdate]() { update_note(existingNote, noteUpdate); });
+    auto existing = existing_note.uri();
+    utils::main_context_call([this, existing, note_update]() {
+      note_mgr().find_by_uri(existing, [this, note_update](NoteBase & note) {
+        update_note(note, note_update);
+      });
+    });
   }
 
 
-  void SyncManager::delete_note_in_main_thread(const Note::Ptr & existingNote)
+  void SyncManager::delete_note_in_main_thread(const NoteBase & existing_note)
   {
     // Note deletion may affect the GUI, so we have to use the
     // delegate to run in the main gtk thread.
-    // To be consistent, any exceptions in the delgate will be caught
-    // and then rethrown in the synchronization thread.
-    utils::main_context_call([this, existingNote]() { delete_note(existingNote); });
+    auto existing = existing_note.uri();
+    utils::main_context_call([this, existing]() {
+      auto note = note_mgr().find_by_uri(existing);
+      if(note) {
+        delete_note(note.value());
+      }
+    });
   }
 
 
-  void SyncManager::update_local_note(const NoteBase::Ptr & localNote, const NoteUpdate & serverNote, NoteSyncType syncType)
+  void SyncManager::update_local_note(NoteBase & local_note, const NoteUpdate & server_note, NoteSyncType sync_type)
   {
     // In each case, update existingNote's content and revision
     try {
-      localNote->load_foreign_note_xml(serverNote.m_xml_content, OTHER_DATA_CHANGED);
+      local_note.load_foreign_note_xml(server_note.m_xml_content, OTHER_DATA_CHANGED);
     }
     catch(...)
     {} // TODO: Handle exception in case that serverNote.XmlContent is invalid XML
-    m_client->set_revision(std::static_pointer_cast<Note>(localNote), serverNote.m_latest_revision);
+    m_client->set_revision(local_note, server_note.m_latest_revision);
 
     // Update dialog's sync status
     if(m_sync_ui != 0) {
-      m_sync_ui->note_synchronized(localNote->get_title(), syncType);
+      m_sync_ui->note_synchronized(local_note.get_title(), sync_type);
     }
   }
 
 
-  NoteBase::Ptr SyncManager::find_note_by_uuid(const Glib::ustring & uuid)
+  NoteBase::ORef SyncManager::find_note_by_uuid(const Glib::ustring & uuid)
   {
-    return note_mgr().find_by_uri("note://gnote/" + uuid);
+    auto note = note_mgr().find_by_uri("note://gnote/" + uuid);
+    if(note) {
+      return std::ref(*note);
+    }
+    return NoteBase::ORef();
   }
 
 
@@ -675,22 +684,23 @@ namespace sync {
   void SyncManager::delete_notes(SyncServer & server)
   {
     try {
-      // Make list of all local notes
-      auto localNotes = note_mgr().get_notes();
-
       // Get all notes currently on server
-      auto serverNotes = server.get_all_note_uuids();
+      auto server_notes = server.get_all_note_uuids();
+      std::vector<NoteBase::Ref> to_delete;
 
       // Delete notes locally that have been deleted on the server
-      for(const NoteBase::Ptr & iter : localNotes) {
-        Note::Ptr note = std::static_pointer_cast<Note>(iter);
+      note_mgr().for_each([this, server_notes=std::move(server_notes), &to_delete](NoteBase & note) {
 	if(m_client->get_revision(note) != -1
-	   && std::find(serverNotes.begin(), serverNotes.end(), note->id()) == serverNotes.end()) {
+	   && std::find(server_notes.begin(), server_notes.end(), note.id()) == server_notes.end()) {
 	  if(m_sync_ui != 0) {
-	    m_sync_ui->note_synchronized(note->get_title(), DELETE_FROM_CLIENT);
+	    m_sync_ui->note_synchronized(note.get_title(), DELETE_FROM_CLIENT);
 	  }
-	  note_mgr().delete_note(note);
+          to_delete.push_back(note);
 	}
+      });
+
+      for(auto note : to_delete) {
+        note_mgr().delete_note(note);
       }
     }
     catch(std::exception & e) {
@@ -705,7 +715,7 @@ namespace sync {
   void SyncManager::create_note(const NoteUpdate & noteUpdate)
   {
     try {
-      NoteBase::Ptr existingNote = note_mgr().create_with_guid(Glib::ustring(noteUpdate.m_title), Glib::ustring(noteUpdate.m_uuid));
+      auto & existingNote = note_mgr().create_with_guid(Glib::ustring(noteUpdate.m_title), Glib::ustring(noteUpdate.m_uuid));
       update_local_note(existingNote, noteUpdate, DOWNLOAD_NEW);
     }
     catch(std::exception & e) {
@@ -717,10 +727,10 @@ namespace sync {
   }
 
 
-  void SyncManager::update_note(const Note::Ptr & existingNote, const NoteUpdate & noteUpdate)
+  void SyncManager::update_note(NoteBase & existing_note, const NoteUpdate & note_update)
   {
     try {
-      update_local_note(existingNote, noteUpdate, DOWNLOAD_MODIFIED);
+      update_local_note(existing_note, note_update, DOWNLOAD_MODIFIED);
     }
     catch(std::exception & e) {
       DBG_OUT("Exception caught in %s: %s\n", __func__, e.what());
@@ -731,10 +741,10 @@ namespace sync {
   }
 
 
-  void SyncManager::delete_note(const Note::Ptr & existingNote)
+  void SyncManager::delete_note(NoteBase & existing_note)
   {
     try {
-      note_mgr().delete_note(existingNote);
+      note_mgr().delete_note(existing_note);
     }
     catch(std::exception & e) {
       DBG_OUT("Exception caught in %s: %s\n", __func__, e.what());
@@ -745,9 +755,12 @@ namespace sync {
   }
 
 
-  void SyncManager::note_save(const Note::Ptr & note)
+  void SyncManager::note_save(const NoteBase & note)
   {
-    utils::main_context_call(sigc::mem_fun(*note, &Note::save));
+    auto uri = note.uri();
+    utils::main_context_call([this, uri] {
+      note_mgr().find_by_uri(uri, [](NoteBase & note) { note.save(); });
+    });
   }
 
 
