@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2012-2014,2017-2021 Aurimas Cernius
+ * Copyright (C) 2012-2014,2017-2021,2023 Aurimas Cernius
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,24 +60,24 @@ namespace sync {
     virtual SyncServiceAddin *get_configured_sync_service();
     virtual void delete_notes_in_main_thread(SyncServer & server);
     void delete_notes(SyncServer & server);
-    virtual void note_save(const Note::Ptr & note);
+    virtual void note_save(const NoteBase & note);
     virtual void create_note_in_main_thread(const NoteUpdate & noteUpdate);
-    virtual void update_note_in_main_thread(const Note::Ptr & existingNote, const NoteUpdate & noteUpdate);
-    virtual void delete_note_in_main_thread(const Note::Ptr & existingNote);
+    virtual void update_note_in_main_thread(const NoteBase & existing_note, const NoteUpdate & note_update);
+    virtual void delete_note_in_main_thread(const NoteBase & existing_note);
     void create_note(const NoteUpdate & noteUpdate);
-    void update_note(const Note::Ptr & existingNote, const NoteUpdate & noteUpdate);
-    void delete_note(const Note::Ptr & existingNote);
+    void update_note(NoteBase & existing_note, const NoteUpdate & note_update);
+    void delete_note(NoteBase & existing_note);
 
     std::unique_ptr<SyncClient> m_client;
     SyncUI::Ptr m_sync_ui;
   private:
-    void handle_note_saved_or_deleted(const NoteBase::Ptr & note);
-    void handle_note_buffer_changed(const NoteBase::Ptr & note);
+    void handle_note_saved_or_deleted(NoteBase & note);
+    void handle_note_buffer_changed(NoteBase & note);
     void update_sync_action();
     void background_sync_checker();
     void set_state(SyncState new_state);
-    void update_local_note(const NoteBase::Ptr & localNote, const NoteUpdate & serverNote, NoteSyncType syncType);
-    NoteBase::Ptr find_note_by_uuid(const Glib::ustring & uuid);
+    void update_local_note(NoteBase & local_note, const NoteUpdate & server_note, NoteSyncType sync_type);
+    NoteBase::ORef find_note_by_uuid(const Glib::ustring & uuid);
     NoteManagerBase & note_mgr();
     void get_synchronized_xml_bits(const Glib::ustring & noteXml, Glib::ustring & title, Glib::ustring & tags, Glib::ustring & content);
     void abort_sync(SyncServer *server);
