@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2013,2017,2019,2021 Aurimas Cernius
+ * Copyright (C) 2013,2017,2019,2021-2022 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #define __TAG_MANAGER_HPP_
 
 
+#include <mutex>
 #include <sigc++/signal.h>
 
 #include <gtkmm/liststore.h>
@@ -67,14 +68,11 @@ private:
   Glib::RefPtr<Gtk::ListStore>     m_tags;
   Glib::RefPtr<Gtk::TreeModelSort> m_sorted_tags;
   // The key for this dictionary is Tag.Name.ToLower ().
-  typedef std::map<Glib::ustring, Gtk::TreeIter> TagMap;
+  typedef std::map<Glib::ustring, Gtk::TreeIter<Gtk::TreeRow>> TagMap;
   TagMap                           m_tag_map;
   typedef std::map<Glib::ustring, Tag::Ptr> InternalMap;
   InternalMap                      m_internal_tags;
   mutable std::mutex               m_locker;
-  
-  sigc::signal<void, Tag::Ptr, const Gtk::TreeIter &> m_signal_tag_added;
-  sigc::signal<void, const Glib::ustring &> m_signal_tag_removed;
 };
 
 }
