@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010,2016-2017 Aurimas Cernius
+ * Copyright (C) 2010,2016-2017,2019,2022-2023 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -78,15 +78,11 @@ namespace gnote {
   {
   }
 
-  void SplitterAction::split(Gtk::TextIter iter, 
-                             Gtk::TextBuffer * buffer)
+  void SplitterAction::split(Gtk::TextIter iter, Gtk::TextBuffer *buffer)
   {
-    Glib::SListHandle<Glib::RefPtr<Gtk::TextTag> > tag_list = iter.get_tags();
-    for(Glib::SListHandle<Glib::RefPtr<Gtk::TextTag> >::const_iterator tag_iter = tag_list.begin();
-        tag_iter != tag_list.end(); ++tag_iter) {
-      const Glib::RefPtr<Gtk::TextTag>& tag(*tag_iter);
-      NoteTag::ConstPtr noteTag = NoteTag::ConstPtr::cast_dynamic(tag);
-      if (noteTag && !noteTag->can_split()) {
+    for(const auto & tag : iter.get_tags()) {
+      auto noteTag = std::dynamic_pointer_cast<const NoteTag>(tag);
+      if(noteTag && !noteTag->can_split()) {
         Gtk::TextIter start = iter;
         Gtk::TextIter end = iter;
 
@@ -125,10 +121,9 @@ namespace gnote {
   int SplitterAction::get_split_offset() const
   {
     int offset = 0;
-    for(std::list<TagData>::const_iterator iter = m_splitTags.begin();
-        iter != m_splitTags.end(); ++iter) {
-      NoteTag::Ptr noteTag = NoteTag::Ptr::cast_dynamic(iter->tag);
-      if (noteTag->get_image()) {
+    for(auto & iter : m_splitTags) {
+      NoteTag::Ptr noteTag = std::dynamic_pointer_cast<NoteTag>(iter.tag);
+      if(noteTag->get_widget()) {
         offset++;
       }
     }
@@ -138,9 +133,7 @@ namespace gnote {
 
   void SplitterAction::apply_split_tag(Gtk::TextBuffer * buffer)
   {
-    for(std::list<TagData>::const_iterator iter = m_splitTags.begin();
-        iter != m_splitTags.end(); ++iter) {
-      const TagData & tag(*iter);
+    for(const auto & tag : m_splitTags) {
       int offset = get_split_offset ();
 
       Gtk::TextIter start = buffer->get_iter_at_offset (tag.start - offset);
@@ -152,9 +145,7 @@ namespace gnote {
 
   void SplitterAction::remove_split_tags(Gtk::TextBuffer *buffer)
   {
-    for(std::list<TagData>::const_iterator iter = m_splitTags.begin();
-        iter != m_splitTags.end(); ++iter) {
-      const TagData & tag(*iter);
+    for(const auto & tag : m_splitTags) {
       Gtk::TextIter start = buffer->get_iter_at_offset (tag.start);
       Gtk::TextIter end = buffer->get_iter_at_offset (tag.end);
       buffer->remove_tag(tag.tag, start, end);
@@ -542,11 +533,9 @@ namespace gnote {
   
 
 
-  InsertBulletAction::InsertBulletAction(int offset, int depth, 
-                                         Pango::Direction direction)
+  InsertBulletAction::InsertBulletAction(int offset, int depth)
     : m_offset(offset)
     , m_depth(depth)
-    , m_direction(direction)
   {
   }
 
@@ -571,8 +560,7 @@ namespace gnote {
     Gtk::TextIter iter = buffer->get_iter_at_offset (m_offset);
     iter = buffer->insert (iter, "\n");
 
-    dynamic_cast<NoteBuffer*>(buffer)->insert_bullet (iter, 
-                                                      m_depth, m_direction);
+    dynamic_cast<NoteBuffer*>(buffer)->insert_bullet(iter, m_depth);
 
     buffer->move_mark (buffer->get_insert(), iter);
     buffer->move_mark (buffer->get_selection_bound(), iter);
@@ -795,13 +783,12 @@ namespace gnote {
     add_undo_action(new ChangeDepthAction(line, direction));
   }
 
-  void UndoManager::on_bullet_inserted(int offset, int depth, 
-                                       Pango::Direction direction)
+  void UndoManager::on_bullet_inserted(int offset, int depth)
   {
     if(m_frozen_cnt) {
       return;
     }
-    add_undo_action(new InsertBulletAction(offset, depth, direction));
+    add_undo_action(new InsertBulletAction(offset, depth));
   }
 
 
