@@ -30,6 +30,7 @@
 #include <glibmm/i18n.h>
 #include <glibmm/stringutils.h>
 #include <gtkmm/label.h>
+#include <gtkmm/urilauncher.h>
 
 #include "sharp/xmlreader.hpp"
 #include "sharp/xmlwriter.hpp"
@@ -64,29 +65,24 @@ namespace gnote {
         uri += "/" + link_id;
       }
 
-      gtk_show_uri_full(parent.gobj(), uri.c_str(), GDK_CURRENT_TIME, nullptr, [](GObject *obj, GAsyncResult *res, gpointer data) {
-        auto parent = static_cast<Gtk::Window*>(data);
-        GError *error = NULL;
-        if(gtk_show_uri_full_finish(parent->gobj(), res, &error)) {
-          return;
+      auto launcher = Gtk::UriLauncher::create(uri);
+      launcher->launch(parent, [launcher, &parent](const Glib::RefPtr<Gio::AsyncResult> & ready) {
+        try {
+          launcher->launch_finish(ready);
         }
-        if(error) {
-          g_error_free(error);
+        catch(const Glib::Error & error) {
+          ERR_OUT(_("Failed to show help: %s"), error.what());
+          Glib::ustring message = _("The \"Gnote Manual\" could "
+                                    "not be found.  Please verify "
+                                    "that your installation has been "
+                                    "completed successfully.");
+          auto dialog = Gtk::make_managed<HIGMessageDialog>(const_cast<Gtk::Window*>(&parent),
+            GTK_DIALOG_DESTROY_WITH_PARENT, Gtk::MessageType::ERROR, Gtk::ButtonsType::OK,
+            _("Help not found"), message);
+          dialog->show();
+          dialog->signal_response().connect([dialog](int) { dialog->hide(); });
         }
-        
-        Glib::ustring message = _("The \"Gnote Manual\" could "
-                                  "not be found.  Please verify "
-                                  "that your installation has been "
-                                  "completed successfully.");
-        auto dialog = Gtk::make_managed<HIGMessageDialog>(parent,
-                                GTK_DIALOG_DESTROY_WITH_PARENT,
-                                Gtk::MessageType::ERROR,
-                                Gtk::ButtonsType::OK,
-                                _("Help not found"),
-                                message);
-        dialog->show();
-        dialog->signal_response().connect([dialog](int) { dialog->hide(); });
-      }, &parent);
+      });
     }
 
 
@@ -94,7 +90,15 @@ namespace gnote {
     {
       if(!url.empty()) {
         DBG_OUT("Opening url '%s'...", url.c_str());
-        gtk_show_uri(parent.gobj(), url.c_str(), GDK_CURRENT_TIME);
+        auto launcher = Gtk::UriLauncher::create(url);
+        launcher->launch(parent, [launcher](const Glib::RefPtr<Gio::AsyncResult> & ready) {
+          try {
+            launcher->launch_finish(ready);
+          }
+          catch(const Glib::Error & error) {
+            ERR_OUT(_("Failed to open URL: %s"), error.what());
+          }
+        });
       }
     }
 
@@ -247,6 +251,17 @@ namespace gnote {
     }
 
 
+    void timeout_add_once(guint interval, std::function<void()> func)
+    {
+      auto f = new std::function<void()>(std::move(func));
+      g_timeout_add_once(interval, [](gpointer data) {
+        auto func = static_cast<std::function<void()>*>(data);
+        (*func)();
+        delete func;
+      }, f);
+    }
+
+
     HIGMessageDialog::HIGMessageDialog(Gtk::Window *parent,
                                        GtkDialogFlags flags, Gtk::MessageType msg_type, 
                                        Gtk::ButtonsType btn_type, const Glib::ustring & header,
@@ -359,6 +374,11 @@ namespace gnote {
       signal_bind().connect(sigc::mem_fun(*this, &LabelFactory::on_bind));
     }
 
+    void LabelFactory::set_text(Gtk::Label & label, const Glib::ustring & text)
+    {
+      label.set_text(text);
+    }
+
     void LabelFactory::on_setup(const Glib::RefPtr<Gtk::ListItem> & item)
     {
       auto label = Gtk::make_managed<Gtk::Label>();
