@@ -1,7 +1,7 @@
 /*
  * gnote
  *
- * Copyright (C) 2010-2015,2017,2019-2023 Aurimas Cernius
+ * Copyright (C) 2010-2015,2017,2019-2024 Aurimas Cernius
  * Copyright (C) 2010 Debarshi Ray
  * Copyright (C) 2009 Hubert Figuiere
  *
@@ -26,7 +26,9 @@
 
 #include <glibmm/i18n.h>
 #include <glibmm/stringutils.h>
-#include <gtkmm/separatormenuitem.h>
+#include <gtkmm/eventcontrollerfocus.h>
+#include <gtkmm/eventcontrollermotion.h>
+#include <gtkmm/gestureclick.h>
 
 #include "sharp/string.hpp"
 #include "debug.hpp"
@@ -55,7 +57,7 @@ namespace gnote {
 
   void NoteRenameWatcher::initialize ()
   {
-    m_title_tag = get_note()->get_tag_table()->lookup("note-title");
+    m_title_tag = get_note().get_tag_table()->lookup("note-title");
   }
 
   void NoteRenameWatcher::shutdown ()
@@ -88,31 +90,25 @@ namespace gnote {
     buffer->signal_erase().connect(
       sigc::mem_fun(*this, &NoteRenameWatcher::on_delete_range));
 
-    get_window()->editor()->signal_focus_out_event().connect(
-      sigc::mem_fun(*this, &NoteRenameWatcher::on_editor_focus_out));
+    auto focus_controller = Gtk::EventControllerFocus::create();
+    focus_controller->signal_leave().connect(sigc::mem_fun(*this, &NoteRenameWatcher::on_editor_focus_out));
+    get_window()->editor()->add_controller(focus_controller);
     get_window()->signal_backgrounded.connect(
       sigc::mem_fun(*this, &NoteRenameWatcher::on_window_backgrounded));
 
-    // FIXME: Needed because we hide on delete event, and
-    // just hide on accelerator key, so we can't use delete
-    // event.  This means the window will flash if closed
-    // with a name clash.
-
     // Clean up title line
     buffer->remove_all_tags (get_title_start(), get_title_end());
     buffer->apply_tag (m_title_tag, get_title_start(), get_title_end());
   }
 
 
-  bool NoteRenameWatcher::on_editor_focus_out(GdkEventFocus *)
+  void NoteRenameWatcher::on_editor_focus_out()
   {
-    // TODO: Duplicated from Update(); refactor instead
-    if (m_editing_title) {
-      changed ();
+    if(m_editing_title) {
+      changed();
       update_note_title(false);
       m_editing_title = false;
     }
-    return false;
   }
 
 
@@ -190,13 +186,10 @@ namespace gnote {
 
   Glib::ustring NoteRenameWatcher::get_unique_untitled()
   {
-    int new_num = manager().get_notes().size();
-    Glib::ustring temp_title;
-
-    while (true) {
+    for(int i = 1;; ++i) {
       // TRANSLATORS: %1 is the placeholder for the number.
-      temp_title = Glib::ustring::compose(_("(Untitled %1)"), ++new_num);
-      if (!manager().find (temp_title)) {
+      auto temp_title = Glib::ustring::compose(_("(Untitled %1)"), i);
+      if(!manager().find(temp_title)) {
         return temp_title;
       }
     }
@@ -206,16 +199,23 @@ namespace gnote {
 
   bool NoteRenameWatcher::update_note_title(bool only_warn)
   {
+    Note & note = get_note();
     Glib::ustring title = get_window()->get_name();
-
-    NoteBase::Ptr existing = manager().find (title);
-    if (existing && (existing != get_note())) {
-      show_name_clash_error (title, only_warn);
+    if(note.get_title() == title) {
       return false;
     }
 
-    DBG_OUT ("Renaming note from %s to %s", get_note()->get_title().c_str(), title.c_str());
-    get_note()->set_title(std::move(title), true);
+    if(auto n = manager().find(title)) {
+      NoteBase &existing = n.value();
+      if(&existing != &note) {
+        show_name_clash_error(title, only_warn);
+      }
+
+      return false;  // same note, same title
+    }
+
+    DBG_OUT("Renaming note from '%s' to '%s'", note.get_title().c_str(), title.c_str());
+    note.set_title(std::move(title), true);
     return true;
   }
 
@@ -240,8 +240,8 @@ namespace gnote {
       m_title_taken_dialog =
         new utils::HIGMessageDialog (parent,
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
-                                     Gtk::MESSAGE_WARNING,
-                                     Gtk::BUTTONS_OK,
+                                     Gtk::MessageType::WARNING,
+                                     Gtk::ButtonsType::OK,
                                      _("Note title taken"),
                                      message);
       m_title_taken_dialog->signal_response().connect(
@@ -329,7 +329,7 @@ namespace gnote {
     if (!get_note()->get_tag_table()->lookup ("gtkspell-misspelled")) {
       NoteTag::Ptr tag = NoteTag::create ("gtkspell-misspelled", NoteTag::CAN_SPELL_CHECK);
       tag->set_can_serialize(false);
-      tag->property_underline() = Pango::UNDERLINE_ERROR;
+      tag->property_underline() = Pango::Underline::ERROR;
       get_note()->get_tag_table()->add (tag);
     }
 
@@ -398,12 +398,8 @@ namespace gnote {
 
     if (tag->property_name() == "gtkspell-misspelled") {
         // Remove misspelled tag for links & title
-      Glib::SListHandle<Glib::RefPtr<const Gtk::TextTag> > tag_list = start_char.get_tags();
-      for(Glib::SListHandle<Glib::RefPtr<const Gtk::TextTag> >::const_iterator tag_iter = tag_list.begin();
-          tag_iter != tag_list.end(); ++tag_iter) {
-        const Glib::RefPtr<const Gtk::TextTag>& atag(*tag_iter);
-        if ((tag != atag) &&
-            !NoteTagTable::tag_is_spell_checkable (atag)) {
+      for(const auto & atag : start_char.get_tags()) {
+        if(tag != atag && !NoteTagTable::tag_is_spell_checkable(atag)) {
           // cancel attempt to add misspelled tag on non-spell-check place
           get_buffer()->signal_apply_tag().emission_stop();
           break;
@@ -512,7 +508,7 @@ namespace gnote {
   
 
   NoteUrlWatcher::NoteUrlWatcher()
-    : m_regex(Glib::Regex::create(URL_REGEX, Glib::REGEX_CASELESS))
+    : m_regex(Glib::Regex::create(URL_REGEX, Glib::Regex::CompileFlags::CASELESS))
   {
   }
 
@@ -524,7 +520,7 @@ namespace gnote {
 
   void NoteUrlWatcher::initialize ()
   {
-    m_url_tag = get_note()->get_tag_table()->get_url_tag();
+    m_url_tag = get_note().get_tag_table()->get_url_tag();
   }
 
 
@@ -549,22 +545,12 @@ namespace gnote {
       s_text_event_connected = true;
     }
 
-    m_click_mark = get_buffer()->create_mark(get_buffer()->begin(), true);
-
     get_buffer()->signal_insert().connect(
       sigc::mem_fun(*this, &NoteUrlWatcher::on_insert_text));
     get_buffer()->signal_apply_tag().connect(
       sigc::mem_fun(*this, &NoteUrlWatcher::on_apply_tag));
     get_buffer()->signal_erase().connect(
       sigc::mem_fun(*this, &NoteUrlWatcher::on_delete_range));
-
-    Gtk::TextView * editor(get_window()->editor());
-    editor->signal_button_press_event().connect(
-      sigc::mem_fun(*this, &NoteUrlWatcher::on_button_press), false);
-    editor->signal_populate_popup().connect(
-      sigc::mem_fun(*this, &NoteUrlWatcher::on_populate_popup));
-    editor->signal_popup_menu().connect(
-      sigc::mem_fun(*this, &NoteUrlWatcher::on_popup_menu), false);
   }
 
   Glib::ustring NoteUrlWatcher::get_url(const Gtk::TextIter & start, const Gtk::TextIter & end)
@@ -675,89 +661,6 @@ namespace gnote {
     }
   }
 
-
-
-  bool NoteUrlWatcher::on_button_press(GdkEventButton *ev)
-  {
-    int x, y;
-    gdouble ev_x, ev_y;
-    gdk_event_get_coords((GdkEvent*)ev, &ev_x, &ev_y);
-
-    get_window()->editor()->window_to_buffer_coords (Gtk::TEXT_WINDOW_TEXT, ev_x, ev_y, x, y);
-    Gtk::TextIter click_iter;
-    get_window()->editor()->get_iter_at_location (click_iter, x, y);
-
-    // Move click_mark to click location
-    get_buffer()->move_mark (m_click_mark, click_iter);
-
-    // Continue event processing
-    return false;
-  }
-
-
-  void NoteUrlWatcher::on_populate_popup(Gtk::Menu *menu)
-  {
-    Gtk::TextIter click_iter = get_buffer()->get_iter_at_mark (m_click_mark);
-    if (click_iter.has_tag (m_url_tag) || click_iter.ends_tag (m_url_tag)) {
-      Gtk::MenuItem *item;
-
-      item = manage(new Gtk::SeparatorMenuItem ());
-      item->show ();
-      menu->prepend (*item);
-
-      item = manage(new Gtk::MenuItem (_("_Copy Link Address"), true));
-      item->signal_activate().connect(
-        sigc::mem_fun(*this, &NoteUrlWatcher::copy_link_activate));
-      item->show ();
-      menu->prepend (*item);
-
-      item = manage(new Gtk::MenuItem (_("_Open Link"), true));
-      item->signal_activate().connect(
-        sigc::mem_fun(*this, &NoteUrlWatcher::open_link_activate));
-      item->show ();
-      menu->prepend (*item);
-    }
-  }
-
-
-  bool NoteUrlWatcher::on_popup_menu()
-  {
-    Gtk::TextIter click_iter = get_buffer()->get_iter_at_mark (get_buffer()->get_insert());
-    get_buffer()->move_mark (m_click_mark, click_iter);
-    return false;
-  }
-
-  void NoteUrlWatcher::open_link_activate()
-  {
-    try {
-      Gtk::TextIter click_iter = get_buffer()->get_iter_at_mark (m_click_mark);
-
-      Gtk::TextIter start, end;
-      m_url_tag->get_extents (click_iter, start, end);
-
-      on_url_tag_activated(*(NoteEditor*)get_window()->editor(), start, end);
-    }
-    catch(const std::exception & e) {
-      ERR_OUT("Failed to open link: %s", e.what());
-    }
-  }
-
-
-  void NoteUrlWatcher::copy_link_activate()
-  {
-    Gtk::TextIter click_iter = get_buffer()->get_iter_at_mark (m_click_mark);
-
-    Gtk::TextIter start, end;
-    m_url_tag->get_extents (click_iter, start, end);
-
-    Glib::ustring url = get_url(start, end);
-
-    Glib::RefPtr<Gtk::Clipboard> clip 
-      = get_window()->editor()->get_clipboard ("CLIPBOARD");
-    clip->set_text(url);
-  }
-
-
   ////////////////////////////////////////////////////////////////////////
 
   ApplicationAddin *AppLinkWatcher::create()
@@ -797,40 +700,41 @@ namespace gnote {
     return m_initialized;
   }
 
-  void AppLinkWatcher::on_note_added(const NoteBase::Ptr & added)
+  void AppLinkWatcher::on_note_added(NoteBase & added)
   {
-    for(auto & note : note_manager().get_notes()) {
-      if(added == note) {
-        continue;
+    note_manager().for_each([this, &added](NoteBase & note) {
+      if(&added == &note) {
+        return;
       }
 
-      if(!contains_text(note, added->get_title())) {
-        continue;
+      if(!contains_text(note, added.get_title())) {
+        return;
       }
 
       // Highlight previously unlinked text
-      auto n = std::static_pointer_cast<Note>(note);
-      auto buffer = n->get_buffer();
+      auto & n = static_cast<Note&>(note);
+      auto buffer = n.get_buffer();
       highlight_in_block(note_manager(), n, buffer->begin(), buffer->end());
-    }
+    });
   }
 
-  void AppLinkWatcher::on_note_deleted(const NoteBase::Ptr & deleted)
+  void AppLinkWatcher::on_note_deleted(NoteBase & deleted)
   {
-    auto link_tag = std::static_pointer_cast<Note>(deleted)->get_tag_table()->get_link_tag();
-    auto broken_link_tag = std::static_pointer_cast<Note>(deleted)->get_tag_table()->get_broken_link_tag();
+    auto tag_table = static_cast<Note&>(deleted).get_tag_table();
+    auto link_tag = tag_table->get_link_tag();
+    auto broken_link_tag = tag_table->get_broken_link_tag();
 
-    for(auto & note : note_manager().get_notes()) {
-      if(deleted == note) {
-        continue;
+    note_manager().for_each([this, &deleted, &link_tag, &broken_link_tag](NoteBase & note) {
+      if(&deleted == &note) {
+        return;
       }
 
-      if(!contains_text(note, deleted->get_title())) {
-        continue;
+      if(!contains_text(note, deleted.get_title())) {
+        return;
       }
 
-      Glib::ustring old_title_lower = deleted->get_title().lowercase();
-      auto buffer = std::static_pointer_cast<Note>(note)->get_buffer();
+      Glib::ustring old_title_lower = deleted.get_title().lowercase();
+      auto buffer = static_cast<Note&>(note).get_buffer();
 
       // Turn all link:internal to link:broken for the deleted note.
       utils::TextTagEnumerator enumerator(buffer, link_tag);
@@ -842,45 +746,44 @@ namespace gnote {
         buffer->remove_tag(link_tag, range.start(), range.end());
         buffer->apply_tag(broken_link_tag, range.start(), range.end());
       }
-    }
+    });
   }
 
-  void AppLinkWatcher::on_note_renamed(const NoteBase::Ptr & renamed, const Glib::ustring & /*old_title*/)
+  void AppLinkWatcher::on_note_renamed(const NoteBase & renamed, const Glib::ustring & /*old_title*/)
   {
-    for(auto & note : note_manager().get_notes()) {
-      if(renamed == note) {
-        continue;
+    note_manager().for_each([this, &renamed](NoteBase & note) {
+      if(&renamed == &note) {
+        return;
       }
 
       // Highlight previously unlinked text
-      if(contains_text(note, renamed->get_title())) {
-        auto n = std::static_pointer_cast<Note>(note);
-        auto buffer = n->get_buffer();
-        highlight_note_in_block(note_manager(), n, std::static_pointer_cast<Note>(renamed), buffer->begin(), buffer->end());
+      if(contains_text(note, renamed.get_title())) {
+        auto & n = static_cast<Note&>(note);
+        auto buffer = n.get_buffer();
+        highlight_note_in_block(note_manager(), n, renamed, buffer->begin(), buffer->end());
       }
-    }
+    });
   }
 
-  bool AppLinkWatcher::contains_text(const NoteBase::Ptr & note, const Glib::ustring & text)
+  bool AppLinkWatcher::contains_text(const NoteBase & note, const Glib::ustring & text)
   {
-    Glib::ustring body = note->text_content().lowercase();
+    Glib::ustring body = const_cast<NoteBase&>(note).text_content().lowercase();
     Glib::ustring match = text.lowercase();
 
     return body.find(match) != Glib::ustring::npos;
   }
 
-  void AppLinkWatcher::highlight_in_block(NoteManagerBase & note_manager, const Note::Ptr & note, const Gtk::TextIter & start, const Gtk::TextIter & end)
+  void AppLinkWatcher::highlight_in_block(NoteManagerBase & note_manager, Note & note, const Gtk::TextIter & start, const Gtk::TextIter & end)
   {
-    TrieHit<NoteBase::WeakPtr>::ListPtr hits = note_manager.find_trie_matches(start.get_slice(end));
-    for(TrieHit<NoteBase::WeakPtr>::List::const_iterator iter = hits->begin(); iter != hits->end(); ++iter) {
-      do_highlight(note_manager, note, **iter, start, end);
+    for(const auto & hit : note_manager.find_trie_matches(start.get_slice(end))) {
+      do_highlight(note_manager, note, hit, start, end);
     }
   }
 
-  void AppLinkWatcher::highlight_note_in_block(NoteManagerBase & note_manager, const Note::Ptr & note, const NoteBase::Ptr & find_note, const Gtk::TextIter & start, const Gtk::TextIter & end)
+  void AppLinkWatcher::highlight_note_in_block(NoteManagerBase & note_manager, Note & note, const NoteBase & find_note, const Gtk::TextIter & start, const Gtk::TextIter & end)
   {
     Glib::ustring buffer_text = start.get_text(end).lowercase();
-    Glib::ustring find_title_lower = find_note->get_title().lowercase();
+    Glib::ustring find_title_lower = find_note.get_title().lowercase();
     int idx = 0;
 
     while (true) {
@@ -889,36 +792,38 @@ namespace gnote {
         break;
 
       auto title_len = find_title_lower.length();
-      TrieHit<NoteBase::WeakPtr> hit(idx, idx + title_len, Glib::ustring(find_title_lower), find_note);
+      TrieHit<Glib::ustring> hit(idx, idx + title_len, Glib::ustring(find_title_lower), Glib::ustring(find_note.uri()));
       do_highlight(note_manager, note, hit, start, end);
 
       idx += title_len;
     }
   }
 
-  void AppLinkWatcher::do_highlight(NoteManagerBase & note_manager, const Note::Ptr & note, const TrieHit<NoteBase::WeakPtr> & hit, const Gtk::TextIter & start, const Gtk::TextIter &)
+  void AppLinkWatcher::do_highlight(NoteManagerBase & note_manager, Note & note, const TrieHit<Glib::ustring> & hit, const Gtk::TextIter & start, const Gtk::TextIter &)
   {
     // Some of these checks should be replaced with fixes to
     // TitleTrie.FindMatches, probably.
-    if(hit.value().expired()) {
-      DBG_OUT("DoHighlight: null pointer error for '%s'." , hit.key().c_str());
+    auto hit_ref = note_manager.find_by_uri(hit.value());
+    if(!hit_ref) {
+      DBG_OUT("do_highlight: missing note '%s'." , hit.key().c_str());
       return;
     }
       
     if(!note_manager.find(hit.key())) {
-      DBG_OUT("DoHighlight: '%s' links to non-existing note." , hit.key().c_str());
+      DBG_OUT("do_highlight: '%s' links to non-existing note." , hit.key().c_str());
       return;
     }
       
-    NoteBase::Ptr hit_note(hit.value());
+    const NoteBase & hit_note = hit_ref.value();
 
-    if(hit.key().lowercase() != hit_note->get_title().lowercase()) { // == 0 if same string
-      DBG_OUT("DoHighlight: '%s' links wrongly to note '%s'." , hit.key().c_str(), hit_note->get_title().c_str());
+    if(hit.key().lowercase() != hit_note.get_title().lowercase()) { // == 0 if same string
+      DBG_OUT("do_highlight: '%s' links wrongly to note '%s'." , hit.key().c_str(), hit_note.get_title().c_str());
       return;
     }
       
-    if(hit_note == note)
+    if(&hit_note == &note) {
       return;
+    }
 
     Gtk::TextIter title_start = start;
     title_start.forward_chars(hit.start());
@@ -933,25 +838,25 @@ namespace gnote {
     }
 
     // Don't create links inside URLs
-    if(note->get_tag_table()->has_link_tag(title_start)) {
+    if(note.get_tag_table()->has_link_tag(title_start)) {
       return;
     }
 
     DBG_OUT("Matching Note title '%s' at %d-%d...", hit.key().c_str(), hit.start(), hit.end());
 
-    auto link_tag = note->get_tag_table()->get_link_tag();
-    note->get_tag_table()->foreach(
-      [note, title_start, title_end](const Glib::RefPtr<Gtk::TextTag> & tag) {
-        remove_link_tag(note, tag, title_start, title_end);
+    auto tag_table = note.get_tag_table();
+    auto link_tag = tag_table->get_link_tag();
+    tag_table->foreach([&note, title_start, title_end](const Glib::RefPtr<Gtk::TextTag> & tag) {
+      remove_link_tag(note, tag, title_start, title_end);
     });
-    note->get_buffer()->apply_tag(link_tag, title_start, title_end);
+    note.get_buffer()->apply_tag(link_tag, title_start, title_end);
   }
 
-  void AppLinkWatcher::remove_link_tag(const Note::Ptr & note, const Glib::RefPtr<Gtk::TextTag> & tag, const Gtk::TextIter & start, const Gtk::TextIter & end)
+  void AppLinkWatcher::remove_link_tag(Note & note, const Glib::RefPtr<Gtk::TextTag> & tag, const Gtk::TextIter & start, const Gtk::TextIter & end)
   {
-    NoteTag::Ptr note_tag = NoteTag::Ptr::cast_dynamic(tag);
+    auto note_tag = std::dynamic_pointer_cast<NoteTag>(tag);
     if(note_tag && note_tag->can_activate()) {
-      note->get_buffer()->remove_tag(note_tag, start, end);
+      note.get_buffer()->remove_tag(note_tag, start, end);
     }
   }
 
@@ -968,8 +873,9 @@ namespace gnote {
 
   void NoteLinkWatcher::initialize ()
   {
-    m_link_tag = get_note()->get_tag_table()->get_link_tag();
-    m_broken_link_tag = get_note()->get_tag_table()->get_broken_link_tag();
+    auto & tag_table = get_note().get_tag_table();
+    m_link_tag = tag_table->get_link_tag();
+    m_broken_link_tag = tag_table->get_broken_link_tag();
   }
 
 
@@ -1001,12 +907,6 @@ namespace gnote {
       sigc::mem_fun(*this, &NoteLinkWatcher::on_delete_range));
   }
 
-
-  void NoteLinkWatcher::do_highlight(const TrieHit<NoteBase::WeakPtr> & hit, const Gtk::TextIter & start, const Gtk::TextIter & end)
-  {
-    AppLinkWatcher::do_highlight(manager(), get_note(), hit, start, end);
-  }
-
   void NoteLinkWatcher::highlight_in_block(const Gtk::TextIter & start,
                                            const Gtk::TextIter & end)
   {
@@ -1055,12 +955,15 @@ namespace gnote {
   void NoteLinkWatcher::on_apply_tag(const Glib::RefPtr<Gtk::TextBuffer::Tag> & tag,
                                      const Gtk::TextIter & start, const Gtk::TextIter &end)
   {
-    if (tag->property_name() != get_note()->get_tag_table()->get_link_tag()->property_name())
+    Glib::ustring tag_name = tag->property_name();
+    Glib::ustring link_tag_name = get_note().get_tag_table()->get_link_tag()->property_name();
+    if(tag_name != link_tag_name)
       return;
-    Glib::ustring link_name = start.get_text (end);
-    NoteBase::Ptr link = manager().find(link_name);
-    if(!link)
-        unhighlight_in_block(start, end);
+    Glib::ustring link_name = start.get_text(end);
+    auto link = manager().find(link_name);
+    if(!link) {
+      unhighlight_in_block(start, end);
+    }
   }
 
 
@@ -1069,12 +972,12 @@ namespace gnote {
                                             const Gtk::TextIter & end)
   {
     Glib::ustring link_name = start.get_text (end);
-    NoteBase::Ptr link = manager().find(link_name);
+    auto link = manager().find(link_name);
 
     if (!link) {
       DBG_OUT("Creating note '%s'...", link_name.c_str());
       try {
-        link = manager().create(Glib::ustring(link_name));
+        link = std::ref(manager().create(Glib::ustring(link_name)));
       } 
       catch(...)
       {
@@ -1082,10 +985,11 @@ namespace gnote {
       }
     }
 
-    Glib::RefPtr<Gtk::TextTag> broken_link_tag = get_note()->get_tag_table()->get_broken_link_tag();
+    Note & note = get_note();
+    Glib::RefPtr<Gtk::TextTag> broken_link_tag = note.get_tag_table()->get_broken_link_tag();
     if(start.starts_tag(broken_link_tag)) {
-      get_note()->get_buffer()->remove_tag(broken_link_tag, start, end);
-      get_note()->get_buffer()->apply_tag(get_note()->get_tag_table()->get_link_tag(), start, end);
+      note.get_buffer()->remove_tag(broken_link_tag, start, end);
+      note.get_buffer()->apply_tag(note.get_tag_table()->get_link_tag(), start, end);
     }
 
     // FIXME: We used to also check here for (link != this.Note), but
@@ -1096,7 +1000,7 @@ namespace gnote {
     // also works around the bug.
     if (link) {
       DBG_OUT ("Opening note '%s' on click...", link_name.c_str());
-      MainWindow::present_default(ignote(), std::static_pointer_cast<Note>(link));
+      MainWindow::present_default(ignote(), static_cast<Note&>(link.value().get()));
       return true;
     }
 
@@ -1124,7 +1028,7 @@ namespace gnote {
 
   void NoteWikiWatcher::initialize ()
   {
-    m_broken_link_tag = get_note()->get_tag_table()->get_broken_link_tag();
+    m_broken_link_tag = get_note().get_tag_table()->get_broken_link_tag();
   }
 
 
@@ -1164,7 +1068,7 @@ namespace gnote {
       Gtk::TextIter end_cpy = start_cpy;
       end_cpy.forward_chars(match.size());
 
-      if(get_note()->get_tag_table()->has_link_tag(start_cpy)) {
+      if(get_note().get_tag_table()->has_link_tag(start_cpy)) {
 	break;
       }
 
@@ -1204,8 +1108,8 @@ namespace gnote {
   void MouseHandWatcher::_init_static()
   {
     if(!s_static_inited) {
-      s_normal_cursor = Gdk::Cursor::create(Gdk::XTERM);
-      s_hand_cursor = Gdk::Cursor::create(Gdk::HAND2);
+      s_normal_cursor = Gdk::Cursor::create("text");
+      s_hand_cursor = Gdk::Cursor::create("pointer");
       s_static_inited = true;
     }
   }
@@ -1230,54 +1134,46 @@ namespace gnote {
   }
   
 
-  void MouseHandWatcher::on_note_opened ()
+  void MouseHandWatcher::on_note_opened()
   {
     Gtk::TextView *editor = get_window()->editor();
-    editor->signal_motion_notify_event()
+    auto motion_ctrl = Gtk::EventControllerMotion::create();
+    motion_ctrl->signal_motion()
       .connect(sigc::mem_fun(*this, &MouseHandWatcher::on_editor_motion), false);
-    editor->signal_key_press_event()
+    editor->add_controller(motion_ctrl);
+    auto & key_ctrl = dynamic_cast<NoteEditor*>(editor)->key_controller();
+    key_ctrl.signal_key_pressed()
       .connect(sigc::mem_fun(*this, &MouseHandWatcher::on_editor_key_press), false);
-    editor->signal_key_release_event()
-      .connect(sigc::mem_fun(*this, &MouseHandWatcher::on_editor_key_release), false);
+    auto click_ctrl = Gtk::GestureClick::create();
+    click_ctrl->set_button(1);
+    click_ctrl->signal_released()
+      .connect([this, click_ctrl](int, double x, double y) {
+        on_button_release(x, y, click_ctrl->get_current_event_state());
+      }, false);
+    editor->add_controller(click_ctrl);
   }
 
-  bool MouseHandWatcher::on_editor_key_press(GdkEventKey* ev)
+  bool MouseHandWatcher::on_editor_key_press(guint keyval, guint, Gdk::ModifierType modifier)
   {
     bool retval = false;
-    auto event = (GdkEvent*)ev;
-    guint keyval;
-    gdk_event_get_keyval(event, &keyval);
 
     switch(keyval) {
-    case GDK_KEY_Shift_L:
-    case GDK_KEY_Shift_R:
-    case GDK_KEY_Control_L:
-    case GDK_KEY_Control_R:
-    {
-      // Control or Shift when hovering over a link
-      // swiches to a bar cursor...
-
-      if (!m_hovering_on_link)
-        break;
-
-      Glib::RefPtr<Gdk::Window> win = get_window()->editor()->get_window (Gtk::TEXT_WINDOW_TEXT);
-      win->set_cursor(s_normal_cursor);
-      break;
-    }
     case GDK_KEY_Return:
     case GDK_KEY_KP_Enter:
     {
-      Gtk::TextIter iter = get_buffer()->get_iter_at_mark (get_buffer()->get_insert());
+      if(Gdk::ModifierType::CONTROL_MASK == (modifier & Gdk::ModifierType::CONTROL_MASK)) {
+        break;
+      }
 
-      Glib::SListHandle<Glib::RefPtr<Gtk::TextTag> > tag_list = iter.get_tags();
-      for(Glib::SListHandle<Glib::RefPtr<Gtk::TextTag> >::const_iterator tag_iter = tag_list.begin();
-          tag_iter != tag_list.end(); ++tag_iter) {
-        const Glib::RefPtr<Gtk::TextTag>& tag(*tag_iter);
+      Gtk::TextIter iter = get_buffer()->get_iter_at_mark(get_buffer()->get_insert());
 
-        if (NoteTagTable::tag_is_activatable (tag)) {
-          retval = gtk_text_tag_event(tag->gobj(), G_OBJECT(get_window()->editor()->gobj()), event, iter.gobj());
-          if (retval) {
-            break;
+      for(const auto & tag : iter.get_tags()) {
+        if(NoteTagTable::tag_is_activatable(tag)) {
+          if(auto note_tag = std::dynamic_pointer_cast<NoteTag>(tag)) {
+            retval = note_tag->activate(*dynamic_cast<NoteEditor*>(get_window()->editor()), iter);
+            if(retval) {
+              break;
+            }
           }
         }
       }
@@ -1290,83 +1186,65 @@ namespace gnote {
   }
 
 
-  bool MouseHandWatcher::on_editor_key_release(GdkEventKey* ev)
+  void MouseHandWatcher::on_editor_motion(double x, double y)
   {
-    bool retval = false;
-    guint keyval;
-    gdk_event_get_keyval((GdkEvent*)ev, &keyval);
-    switch(keyval) {
-    case GDK_KEY_Shift_L:
-    case GDK_KEY_Shift_R:
-    case GDK_KEY_Control_L:
-    case GDK_KEY_Control_R:
-    {
-      if (!m_hovering_on_link)
-        break;
-
-      Glib::RefPtr<Gdk::Window> win = get_window()->editor()->get_window (Gtk::TEXT_WINDOW_TEXT);
-      win->set_cursor(s_hand_cursor);
-      break;
-    }
-    default:
-      break;
-    }
-    return retval;
-  }
-
-
-  bool MouseHandWatcher::on_editor_motion(GdkEventMotion *)
-  {
-    bool retval = false;
-
-    int pointer_x, pointer_y;
-    Gdk::ModifierType pointer_mask;
-
-    get_window()->editor()->Gtk::Widget::get_window()->get_pointer (pointer_x,
-                                                                  pointer_y,
-                                                                  pointer_mask);
-
     bool hovering = false;
+    auto editor = get_window()->editor();
 
     // Figure out if we're on a link by getting the text
     // iter at the mouse point, and checking for tags that
     // start with "link:"...
 
     int buffer_x, buffer_y;
-    get_window()->editor()->window_to_buffer_coords (Gtk::TEXT_WINDOW_WIDGET,
-                                        pointer_x, pointer_y,
-                                                      buffer_x, buffer_y);
+    editor->window_to_buffer_coords(Gtk::TextWindowType::WIDGET, x, y, buffer_x, buffer_y);
 
     Gtk::TextIter iter;
-    get_window()->editor()->get_iter_at_location (iter, buffer_x, buffer_y);
+    editor->get_iter_at_location(iter, buffer_x, buffer_y);
 
-    Glib::SListHandle<Glib::RefPtr<Gtk::TextTag> > tag_list = iter.get_tags();
-    for(Glib::SListHandle<Glib::RefPtr<Gtk::TextTag> >::const_iterator tag_iter = tag_list.begin();
-        tag_iter != tag_list.end(); ++tag_iter) {
-      const Glib::RefPtr<Gtk::TextTag>& tag(*tag_iter);
-
-      if (NoteTagTable::tag_is_activatable (tag)) {
+    for(const auto & tag : iter.get_tags()) {
+      if(NoteTagTable::tag_is_activatable(tag)) {
         hovering = true;
         break;
       }
     }
 
-    // Don't show hand if Shift or Control is pressed
-    bool avoid_hand = (pointer_mask & (Gdk::SHIFT_MASK |
-                                       Gdk::CONTROL_MASK)) != 0;
-
-    if (hovering != m_hovering_on_link) {
+    if(hovering != m_hovering_on_link) {
       m_hovering_on_link = hovering;
 
-      Glib::RefPtr<Gdk::Window> win = get_window()->editor()->get_window(Gtk::TEXT_WINDOW_TEXT);
-      if (hovering && !avoid_hand) {
-        win->set_cursor(s_hand_cursor);
+      if(hovering) {
+        editor->set_cursor(s_hand_cursor);
       }
       else {
-        win->set_cursor(s_normal_cursor);
+        editor->set_cursor(s_normal_cursor);
+      }
+    }
+  }
+
+
+  void MouseHandWatcher::on_button_release(double x, double y, Gdk::ModifierType state)
+  {
+    if(Gdk::ModifierType::CONTROL_MASK == (state & Gdk::ModifierType::CONTROL_MASK)) {
+      return;
+    }
+    if(Gdk::ModifierType::SHIFT_MASK == (state & Gdk::ModifierType::SHIFT_MASK)) {
+      return;
+    }
+
+    auto editor = get_window()->editor();
+    int buffer_x, buffer_y;
+    editor->window_to_buffer_coords(Gtk::TextWindowType::WIDGET, x, y, buffer_x, buffer_y);
+    Gtk::TextIter iter;
+    editor->get_iter_at_location(iter, buffer_x, buffer_y);
+
+    for(const auto & tag : iter.get_tags()) {
+      if(NoteTagTable::tag_is_activatable(tag)) {
+        if(auto note_tag = std::dynamic_pointer_cast<NoteTag>(tag)) {
+          if(note_tag->activate(*dynamic_cast<NoteEditor*>(get_window()->editor()), iter)) {
+            break;
+          }
+        }
       }
     }
-    return retval;
   }
 
   ////////////////////////////////////////////////////////////////////////
@@ -1381,14 +1259,12 @@ namespace gnote {
 
   void NoteTagsWatcher::initialize ()
   {
+    Note & note = get_note();
 #ifdef DEBUG
-    m_on_tag_added_cid = get_note()->signal_tag_added.connect(
-      sigc::mem_fun(*this, &NoteTagsWatcher::on_tag_added));
-    m_on_tag_removing_cid = get_note()->signal_tag_removing.connect(
-      sigc::mem_fun(*this, &NoteTagsWatcher::on_tag_removing));
+    m_on_tag_added_cid = note.signal_tag_added.connect(sigc::mem_fun(*this, &NoteTagsWatcher::on_tag_added));
+    m_on_tag_removing_cid = note.signal_tag_removing.connect(sigc::mem_fun(*this, &NoteTagsWatcher::on_tag_removing));
 #endif
-    m_on_tag_removed_cid = get_note()->signal_tag_removed.connect(
-      sigc::mem_fun(*this, &NoteTagsWatcher::on_tag_removed));      
+    m_on_tag_removed_cid = note.signal_tag_removed.connect(sigc::mem_fun(*this, &NoteTagsWatcher::on_tag_removed));
   }
 
 
@@ -1403,10 +1279,10 @@ namespace gnote {
   void NoteTagsWatcher::on_note_opened ()
   {
     // FIXME: Just for kicks, spit out the current tags
-    DBG_OUT ("%s tags:", get_note()->get_title().c_str());
-//    foreach (const Tag::Ptr & tag, get_note()->tags()) {
-//      DBG_OUT ("\t%s", tag->name().c_str());
-//    }
+    DBG_OUT ("%s tags:", get_note().get_title().c_str());
+    for(const auto & tag : get_note().get_tags()) {
+      DBG_OUT ("\t%s", tag->name().c_str());
+    }
   }
 
 #ifdef DEBUG
@@ -1423,7 +1299,7 @@ namespace gnote {
 #endif
 
 
-  void NoteTagsWatcher::on_tag_removed(const NoteBase::Ptr&, const Glib::ustring& tag_name)
+  void NoteTagsWatcher::on_tag_removed(const NoteBase&, const Glib::ustring& tag_name)
   {
     Tag::Ptr tag = manager().tag_manager().get_tag(tag_name);
     DBG_OUT ("Watchers.OnTagRemoved popularity count: %d", tag ? tag->popularity() : 0);
